package api.common.util;


import org.apache.sshd.client.SshClient;
import org.apache.sshd.client.channel.ChannelExec;
import org.apache.sshd.client.channel.ClientChannelEvent;
import org.apache.sshd.client.future.AuthFuture;
import org.apache.sshd.client.session.ClientSession;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Collections;

/**
 * https://github.com/apache/mina-sshd
 * https://github.com/apache/mina-sshd/blob/master/docs/client-setup.md
 * <!-- ssh -->
 * <dependency>
 * <groupId>org.apache.sshd</groupId>
 * <artifactId>sshd-core</artifactId>
 * <version>${sshd-core.version}</version>
 * </dependency>
 */
public class SshUtil {

    private static final int port = 22;

    /**
     *
     * @param hostname  主机名或 IP
     * @param username  用户名
     * @param password  密码
     * @param command   命令
     * @return  执行结果
     * @throws IOException  异常
     */
    public static String execute(String hostname, String username, String password, String command) throws IOException {
        String result;

        SshClient client = SshClient.setUpDefaultClient();
        client.start();
        // using the client for multiple sessions...
        try (ClientSession session = client.connect(username, hostname, port).verify(10000).getSession()) {
            session.addPasswordIdentity(password); // for password-based authentication
            AuthFuture verify = session.auth().verify(10000);
            if (verify.isFailure()) {
                throw new RuntimeException("------- SSH 用户名密码验证失败！");
            }
            ChannelExec execChannel = session.createExecChannel(command);
            // 创建输出流
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream(2048);
            execChannel.setOut(byteArrayOutputStream);
            execChannel.open();
            execChannel.waitFor(Collections.singleton(ClientChannelEvent.CLOSED), 0);
            execChannel.close();
            // 结果写入
            result = byteArrayOutputStream.toString();
            byteArrayOutputStream.close();
        }
        client.stop();
        return result;
    }


}
