package com.css.simulation.resource.model.ctrl;


import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.param.model.ConfigPageParam;
import api.common.pojo.po.model.ConfigPO;
import api.common.pojo.vo.model.ConfigSensorVO;
import api.common.pojo.vo.model.ConfigVO;
import api.common.util.ObjectUtil;
import com.css.simulation.resource.model.service.ConfigService;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.List;
import java.util.Map;

@Controller
@RequestMapping("/config")
public class ConfigCtrl {

    @Autowired
    ConfigService configService;

    /**
     * 获取列表
     */
    @RequestMapping("/getConfigPageList")
    @ResponseBody
    public ResponseBodyVO<PageInfo<ConfigVO>> getConfigPageList(@RequestBody @Validated ConfigPageParam pageParam){
        ResponseBodyVO<PageInfo<ConfigVO>> response = new ResponseBodyVO<PageInfo<ConfigVO>>(ResponseBodyVO.Response.SUCCESS);
        response.setInfo(configService.getConfigPageList(pageParam));
        return response;
    }


    /**
     * 根据id获取详情
     */
    @RequestMapping("/getConfigInfo")
    @ResponseBody
    public ResponseBodyVO<ConfigVO> getConfigInfo(@RequestBody ConfigVO configVO){
        ResponseBodyVO<ConfigVO> response = new ResponseBodyVO<ConfigVO>(ResponseBodyVO.Response.SUCCESS);
        response.setInfo(configService.getConfigInfo(configVO));
        return response;
    }

    /**
     * 新增、修改
     */
    @RequestMapping("/saveConfig")
    @ResponseBody
    public ResponseBodyVO<ConfigPO> saveConfig(@RequestBody ConfigVO configVO){
        if(ObjectUtil.isNull(configVO)){
            return new ResponseBodyVO(false, 500, "参数必传！",null);
        }
        //端口重复校验
        Map<String, List<ConfigSensorVO>> configSensors = configVO.getConfigSensors();
        if(ObjectUtil.isNotNull(configSensors)){
            List<ConfigSensorVO> configSensorVOS = configSensors.get("ogt");//完美传感器
            if(ObjectUtil.isNotNull(configSensorVOS)){
                long count = configSensorVOS.stream().mapToInt(ConfigSensorVO::getSensorPort).distinct().count();
                if(configSensorVOS.size() != count){
                    return new ResponseBodyVO(false, 500, "端口重复！",null);
                }
            }
        }
        ResponseBodyVO<ConfigPO> response = new ResponseBodyVO<ConfigPO>(ResponseBodyVO.Response.SUCCESS);
        ConfigPO po = configService.saveConfig(configVO);
        if(ObjectUtil.isNull(po.getId())){
            return new ResponseBodyVO(false, 500, "配置名称重复！",null);
        }
        response.setInfo(po);
        return response;
    }

    /**
     * 详情分享
     */
    @RequestMapping("/shareConfigByInfo")
    @ResponseBody
    public ResponseBodyVO<ConfigPO> shareConfigByInfo(@RequestBody ConfigVO configVO){
        if(ObjectUtil.isNull(configVO)){
            return new ResponseBodyVO(false, 500, "参数必传！",null);
        }
        ResponseBodyVO<ConfigPO> response = new ResponseBodyVO<ConfigPO>(ResponseBodyVO.Response.SUCCESS);
        ConfigPO po = configService.shareConfigByInfo(configVO);
        if(ObjectUtil.isNull(po.getId())){
            return new ResponseBodyVO(false, 500, "配置名称重复！",null);
        }
        response.setInfo(po);
        return response;
    }

    /**
     * 列表分享
     */
    @RequestMapping("/shareConfigById")
    @ResponseBody
    public ResponseBodyVO<ConfigPO> shareConfigById(@RequestBody ConfigVO configVO){
        if(ObjectUtil.isNull(configVO) || ObjectUtil.isNull(configVO.getId())){
            return new ResponseBodyVO(false, 500, "参数必传！",null);
        }
        ResponseBodyVO<ConfigPO> response = new ResponseBodyVO<ConfigPO>(ResponseBodyVO.Response.SUCCESS);
        ConfigPO po = configService.shareConfigById(configVO);
        if(ObjectUtil.isNull(po.getId())){
            return new ResponseBodyVO(false, 500, "配置名称重复！",null);
        }
        response.setInfo(po);
        return response;
    }

    /**
     * 删除
     */
    @RequestMapping("/delConfigById")
    @ResponseBody
    public ResponseBodyVO delConfigById(@RequestBody ConfigVO configVO){
        int i = configService.delConfigById(configVO);
        if(i>0){
            return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS);
        }else{
            return new ResponseBodyVO(false, 500, "删除失败！",null);
        }
    }
}
