package com.css.simulation.resource.model.ctrl;


import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.param.model.GpsParam;
import api.common.pojo.po.model.GpsPO;
import api.common.pojo.vo.model.GpsVO;
import api.common.util.ObjectUtil;
import com.css.simulation.resource.model.service.GpsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.List;

@Controller
@RequestMapping("/gps")
public class GpsCtrl {

    @Autowired
    GpsService gpsService;

    /**
     * 获取列表
     */
    @RequestMapping("/getGpsList")
    @ResponseBody
    public ResponseBodyVO<List<GpsVO>> getGpsList(@RequestBody GpsParam gpsParam){
        ResponseBodyVO<List<GpsVO>> response = new ResponseBodyVO<List<GpsVO>>(ResponseBodyVO.Response.SUCCESS);
        response.setInfo(gpsService.getGpsList(gpsParam));
        return response;
    }

    /**
     * 获取私有列表
     */
    @RequestMapping("/getMyGpsList")
    @ResponseBody
    public ResponseBodyVO<List<GpsVO>> getMyCameraList(@RequestBody GpsParam gpsParam){
        ResponseBodyVO<List<GpsVO>> response = new ResponseBodyVO<List<GpsVO>>(ResponseBodyVO.Response.SUCCESS);
        response.setInfo(gpsService.getMyGpsList(gpsParam));
        return response;
    }

    /**
     * 根据id获取详情
     */
    @RequestMapping("/getGpsInfo")
    @ResponseBody
    public ResponseBodyVO<GpsVO> getGpsInfo(@RequestBody GpsParam gpsParam){
        ResponseBodyVO<GpsVO> response = new ResponseBodyVO<GpsVO>(ResponseBodyVO.Response.SUCCESS);
        response.setInfo(gpsService.getGpsInfo(gpsParam));
        return response;
    }

    /**
     * 新增、修改、另存为
     */
    @RequestMapping("/saveGps")
    @ResponseBody
    public ResponseBodyVO<GpsPO> saveGps(@RequestBody GpsPO gpsPO){
        if(ObjectUtil.isNull(gpsPO)){
            return new ResponseBodyVO(false, 500, "参数必传！",null);
        }
        ResponseBodyVO<GpsPO> response = new ResponseBodyVO<GpsPO>(ResponseBodyVO.Response.SUCCESS);
        GpsPO po = gpsService.saveGps(gpsPO);
        if(ObjectUtil.isNull(po.getId())){
            return new ResponseBodyVO(false, 500, "传感器名称重复！",null);
        }
        response.setInfo(po);
        return response;
    }

    /**
     * 分享
     */
    @RequestMapping("/shareGps")
    @ResponseBody
    public ResponseBodyVO<GpsPO> shareGps(@RequestBody GpsPO gpsPO){
        if(ObjectUtil.isNull(gpsPO)){
            return new ResponseBodyVO(false, 500, "参数必传！",null);
        }
        ResponseBodyVO<GpsPO> response = new ResponseBodyVO<GpsPO>(ResponseBodyVO.Response.SUCCESS);
        GpsPO po = gpsService.shareGps(gpsPO);
        if(ObjectUtil.isNull(po.getId())){
            return new ResponseBodyVO(false, 500, "传感器名称重复！",null);
        }
        response.setInfo(po);
        return response;
    }

    /**
     * 删除
     */
    @RequestMapping("/delGpsById")
    @ResponseBody
    public ResponseBodyVO delGpsById(@RequestBody GpsPO gpsPO){
        int i = gpsService.delGpsById(gpsPO);
        if(i>0){
            return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS);
        }else{
            return new ResponseBodyVO(false, 500, "删除失败！",null);
        }
    }
}
