package com.css.simulation.resource.model.ctrl;


import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.param.model.VehicleParam;
import api.common.pojo.po.model.VehiclePO;
import api.common.pojo.vo.model.VehicleVO;
import api.common.util.ObjectUtil;
import com.css.simulation.resource.model.service.VehicleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.List;

@Controller
@RequestMapping("/vehicle")
public class VehicleCtrl {

    @Autowired
    VehicleService vehicleService;

    /**
     * 获取列表
     */
    @RequestMapping("/getVehicleList")
    @ResponseBody
    public ResponseBodyVO<List<VehicleVO>> getVehicleList(@RequestBody VehicleParam vehicleParam){
        ResponseBodyVO<List<VehicleVO>> response = new ResponseBodyVO<List<VehicleVO>>(ResponseBodyVO.Response.SUCCESS);
        response.setInfo(vehicleService.getVehicleList(vehicleParam));
        return response;
    }

    /**
     * 获取私有列表
     */
    @RequestMapping("/getMyVehicleList")
    @ResponseBody
    public ResponseBodyVO<List<VehicleVO>> getMyVehicleList(@RequestBody VehicleParam vehicleParam){
        ResponseBodyVO<List<VehicleVO>> response = new ResponseBodyVO<List<VehicleVO>>(ResponseBodyVO.Response.SUCCESS);
        response.setInfo(vehicleService.getMyVehicleList(vehicleParam));
        return response;
    }

    /**
     * 根据id获取详情
     */
    @RequestMapping("/getVehicleInfo")
    @ResponseBody
    public ResponseBodyVO<VehicleVO> getVehicleInfo(@RequestBody VehicleParam vehicleParam){
        ResponseBodyVO<VehicleVO> response = new ResponseBodyVO<VehicleVO>(ResponseBodyVO.Response.SUCCESS);
        response.setInfo(vehicleService.getVehicleInfo(vehicleParam));
        return response;
    }

    /**
     * 新增、修改、另存为
     */
    @RequestMapping("/saveVehicle")
    @ResponseBody
    public ResponseBodyVO<VehiclePO> saveVehicle(@RequestBody VehicleVO vehicleVO){
        if(ObjectUtil.isNull(vehicleVO)){
            return new ResponseBodyVO(false, 500, "参数必传！",null);
        }
        ResponseBodyVO<VehiclePO> response = new ResponseBodyVO<VehiclePO>(ResponseBodyVO.Response.SUCCESS);
        VehiclePO po = vehicleService.saveVehicle(vehicleVO);
        if(ObjectUtil.isNull(po.getId())){
            return new ResponseBodyVO(false, 500, "车辆名称重复！",null);
        }
        response.setInfo(po);
        return response;
    }

    /**
     * 分享
     */
    @RequestMapping("/shareVehicle")
    @ResponseBody
    public ResponseBodyVO<VehiclePO> shareVehicle(@RequestBody VehicleVO vehicleVO){
        if(ObjectUtil.isNull(vehicleVO)){
            return new ResponseBodyVO(false, 500, "参数必传！",null);
        }
        ResponseBodyVO<VehiclePO> response = new ResponseBodyVO<VehiclePO>(ResponseBodyVO.Response.SUCCESS);
        VehiclePO po = vehicleService.shareVehicle(vehicleVO);
        if(ObjectUtil.isNull(po.getId())){
            return new ResponseBodyVO(false, 500, "车辆名称重复！",null);
        }
        response.setInfo(po);
        return response;
    }

    /**
     * 删除
     */
    @RequestMapping("/delVehicleById")
    @ResponseBody
    public ResponseBodyVO delVehicleById(@RequestBody VehiclePO vehiclePO){
        int i = vehicleService.delVehicleById(vehiclePO);
        if(i>0){
            return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS);
        }else{
            return new ResponseBodyVO(false, 500, "删除失败！",null);
        }
    }
}
