package com.css.simulation.resource.project.impl;

import api.common.pojo.common.PageVO;
import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.constants.DictConstants;
import api.common.pojo.param.KafkaParameter;
import api.common.pojo.param.MinioParameter;
import api.common.pojo.param.project.ProjectTaskParam;
import api.common.pojo.param.project.SceneScoreParam;
import api.common.pojo.param.project.SimulationManualProjectKafkaParam;
import api.common.pojo.param.project.SimulationManualProjectParam;
import api.common.pojo.param.system.DictParam;
import api.common.pojo.po.algorithm.AlgorithmPO;
import api.common.pojo.po.model.ConfigPO;
import api.common.pojo.po.model.ConfigSensorPO;
import api.common.pojo.po.model.VehiclePO;
import api.common.pojo.po.project.*;
import api.common.pojo.po.scene.ScenePackagePO;
import api.common.pojo.vo.algorithm.AlgorithmVO;
import api.common.pojo.vo.project.*;
import api.common.pojo.vo.scene.RunProjectVO;
import api.common.util.JsonUtil;
import api.common.util.ObjectUtil;
import api.common.util.StringUtil;
import api.common.util.TimeUtil;
import com.css.simulation.resource.common.utils.AuthUtil;
import com.css.simulation.resource.common.utils.PageUtil;
import com.css.simulation.resource.feign.AlgoPlatformService;
import com.css.simulation.resource.feign.FileDownService;
import com.css.simulation.resource.project.constants.ProjectConstants;
import com.css.simulation.resource.project.enums.ProjectRunStateEnum;
import com.css.simulation.resource.project.enums.SceneTypeEnum;
import com.css.simulation.resource.project.feign.KafkaService;
import com.css.simulation.resource.project.mapper.SimulationMptFirstTargetScoreMapper;
import com.css.simulation.resource.project.mapper.SimulationMptLastTargetScoreMapper;
import com.css.simulation.resource.project.mapper.SimulationProjectMapper;
import com.css.simulation.resource.project.mapper.SimulationProjectTaskMapper;
import com.css.simulation.resource.project.service.SimulationProjectService;
import com.css.simulation.resource.system.service.DictService;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.github.pagehelper.PageInfo;
import com.itextpdf.text.*;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;
import feign.Response;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.server.ServletServerHttpRequest;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.net.URLEncoder;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.List;

@Service
@Slf4j
public class SimulationProjectServiceImpl implements SimulationProjectService {

    @Autowired
    private SimulationProjectMapper simulationProjectMapper;

    @Autowired
    private SimulationProjectTaskMapper simulationProjectTaskMapper;

    @Autowired
    private SimulationMptFirstTargetScoreMapper simulationMptFirstTargetScoreMapper;

    @Autowired
    private SimulationMptLastTargetScoreMapper simulationMptLastTargetScoreMapper;

    @Resource
    private KafkaService kafkaService;

    @Autowired
    DictService dictService;

    @Resource
    AlgoPlatformService algoPlatformService;

    @Resource
    FileDownService fileDownService;

    private String[] dateFmtArr= new String[]{"yyyyMMdd", "yyyy-MM-dd HH:mm:ss","yyyy-MM-dd"};

    @Override
    public ResponseBodyVO addOrUpdateProject(SimulationManualProjectParam param) {

        if(isEmpty(param.getProjectName())){
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE,"工作名称不能为空");
        }
        SimulationManualProjectPo po = convertParamToPo(param);

        if(isEmpty(param.getId())){

            //工作名称一样的的不能创建
            List<SimulationManualProjectPo> simulationManualProjectPos = simulationProjectMapper.selectProjectByName(param);
            if(!isEmpty(simulationManualProjectPos)){
                return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE,"工作名称已经存在，请修改后再保存");
            }


            po.createPo(AuthUtil.getCurrentUserId());
            //生成id
            createProjectId(po);

            int add = simulationProjectMapper.add(po);
            if(add > 0){
                //第三方算法存到算发表(索为)
//                insertOtherAlgorithm(param);

                return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,ResponseBodyVO.Response.SUCCESS.getMessage(),po.getId());
            }
            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"添加失败");
        }else{
            //TODO 删除逻辑有待商议
            //获取工作状态，仅未开始的才可以修改信息
            SimulationManualProjectPo simulationManualProjectPo = simulationProjectMapper.selectProjectById(param);
            if(!ProjectRunStateEnum.NOT_START.getCode().equals(simulationManualProjectPo.getNowRunState())){
                return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"当前状态不支持修改");
            }

            po.updatePo(AuthUtil.getCurrentUserId());
            int update = simulationProjectMapper.update(po);
            if(update > 0){
                //第三方算法存到算发表(索为)
//                insertOtherAlgorithm(param);
                return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,ResponseBodyVO.Response.SUCCESS.getMessage(),po.getId());
            }

            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"修改失败");
        }

    }

    @Override
    public ResponseBodyVO selectProject(SimulationManualProjectParam param) {

        if(!isEmpty(param.getCreateStartDate())){
            String createStartDate = param.getCreateStartDate();
            Date startDate = getDate(createStartDate + " 00:00:00",1);
            if(startDate == null){
                return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
            }
            param.setCreateTimeStart(startDate);
        }
        if(!isEmpty(param.getCreateEndDate())){
            String createEndDate = param.getCreateEndDate();
            Date endDate = getDate(createEndDate + " 23:59:59",1);
            if(endDate == null){
                return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
            }
            param.setCreateTimeEnd(endDate);
        }

        if(!isEmpty(param.getFinishDateStart())){
            String finishDateStart = param.getFinishDateStart();
            Date startDate = getDate(finishDateStart + " 00:00:00",1);
            if(startDate == null){
                return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
            }
            param.setFinishTimeStart(startDate);

        }
        if(!isEmpty(param.getFinishDateEnd())){
            String finishDateEnd = param.getFinishDateEnd();
            Date enddate = getDate(finishDateEnd + " 23:59:59",1);
            if(enddate == null){
                return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
            }
            param.setFinishTimeEnd(enddate);
        }

        //查询有效数据
        param.setIsDeleted("0");

        setPage(param.getCurrentPage()==null?1:param.getCurrentPage(), param.getPageSize()==null?10:param.getPageSize());
        List<SimulationManualProjectVo> vos = simulationProjectMapper.selectProjectList(param);
        PageInfo<SimulationManualProjectVo> pageInfo = new PageInfo(vos);

        for(SimulationManualProjectVo p : pageInfo.getList()){
            convertPoToVo(p);
        }

        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,pageInfo);
    }

    @Override
    public ResponseBodyVO selectProjectById(SimulationManualProjectParam param) {
        if(isEmpty(param.getId())){
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
        }
        SimulationManualProjectPo po = simulationProjectMapper.selectProjectById(param);
        if(po == null){
            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"没有获取到数据");
        }

        SimulationManualProjectSingleVo vo = new SimulationManualProjectSingleVo();
        convertPoToVo(po, vo);

        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,vo);

    }


    @Override
    public ResponseBodyVO deleteProjectByids(SimulationManualProjectParam param) {

        String ids = param.getIds();
        if(isEmpty(ids)){
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
        }

        String[] idArr = ids.split(",");

        //未执行和已中止的可以删除
        List<SimulationManualProjectPo> pos = simulationProjectMapper.selectProjectNowRunState(idArr);
        for(SimulationManualProjectPo p : pos){
            if(!isEmpty(p.getNowRunState()) && !ProjectRunStateEnum.NOT_START.getCode().equals(p.getNowRunState()) && !ProjectRunStateEnum.DISCONTINUE.getCode().equals(p.getNowRunState())){
                return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"数据不支持删除");
            }
        }

        int i = simulationProjectMapper.deleteProject(idArr);
        if(i > 0){
            return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS);
        }

        return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"删除失败");
    }

    @SneakyThrows
    @Override
    public ResponseBodyVO updateProjectNowRunState(SimulationManualProjectParam param) {
        log.info("updateProjectNowRunState_id:"+param.getId());
        log.info("updateProjectNowRunState_nowRunState:"+param.getNowRunState());

        SimulationManualProjectPo po = simulationProjectMapper.selectProjectById(param);

        if(po == null){
            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"数据没有找到");
        }

        if(isEmpty(param.getId()) || isEmpty(param.getNowRunState())){
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
        }
        if("20".equals(param.getNowRunState())){
            //设置开始时间
            param.setStartTime(new Date());

            //Kafka推送消息
            projectRunToKafka(po);

        }else if("30".equals(param.getNowRunState())){
            //设置完成时间
            param.setFinishTime(new Date());

        }
        int i = simulationProjectMapper.updateProjectNowRunState(param);
        if(i > 0){
            return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS);
        }
        return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE);


    }

    private void projectRunToKafka(SimulationManualProjectPo po) throws JsonProcessingException {
        SimulationManualProjectKafkaParam kafkaParam = new SimulationManualProjectKafkaParam();
        kafkaParam.setProjectId(po.getId());
        kafkaParam.setAlgorithmId(po.getAlgorithm());
        kafkaParam.setVehicleConfigId(po.getVehicle());
        kafkaParam.setScenePackageId(po.getScene());
        kafkaParam.setMaxSimulationTime(po.getMaxSimulationTime());
        kafkaParam.setParallelism(Integer.valueOf(po.getParallelism()));
        KafkaParameter kafkaParameter = new KafkaParameter();
        kafkaParameter.setTopic(ProjectConstants.RUN_TASK_TOPIC);
        String data = JsonUtil.beanToJson(kafkaParam);
        kafkaParameter.setData(data);
        log.info("推送消息到kafka:"+data);
        kafkaService.send(kafkaParameter);
    }

    @Override
    public ResponseBodyVO selectProjectDetailsById(SimulationManualProjectParam param) {

        if(isEmpty(param.getId())){
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
        }
        ProjectDetailsVo projectDetailsVo = new ProjectDetailsVo();

        //项目基本信息
        SimulationManualProjectPo po = simulationProjectMapper.selectProjectById(param);
        if(po == null){
            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE, "没有获取到数据");
        }

        //评测等级
        String s = getEvaluationLevel(po);

        //算法配置
        AlgorithmPO algorithmBaseInfoVo = getAlgorithmInfo(po);

        //车辆配置
        VehiclePO vehicleBaseInfoVo = null;
        String vehicleConfigId = po.getVehicle();

        ConfigPO configPO = new ConfigPO();
        configPO.setId(vehicleConfigId);
        List<ConfigPO> configVehicleVOS = simulationProjectMapper.selectConfigVehicle(configPO);
        if(!isEmpty(configVehicleVOS)){
            ConfigPO configVehicleVO = configVehicleVOS.get(0);
            List<VehiclePO> vehiclePOS = simulationProjectMapper.selectVehicleBaseInfoById(configVehicleVO.getVehicleId());
            if(!isEmpty(vehiclePOS)){
                vehicleBaseInfoVo = vehiclePOS.get(0);
            }

        }
        /*String vehicleImage = vehicleBaseInfoVo.getVehicleImage();
        if(!isEmpty(vehicleImage)){
            vehicleBaseInfoVo.setVehicleImage(vehicleImage.replaceFirst("Front side","Top"));
        }*/

        List<ConfigSensorPO> vehicleSensorVos = simulationProjectMapper.selectVehicleSensor(vehicleConfigId);

        List<SensorBaseInfoVo> sensorCameraList = new ArrayList<>();
        List<SensorBaseInfoVo> sensorOgtList = new ArrayList<>();
        List<SensorBaseInfoVo> sensorLidarList = new ArrayList<>();
        List<SensorBaseInfoVo> sensorRadarList = new ArrayList<>();
        List<SensorBaseInfoVo> sensorGpsList = new ArrayList<>();
        for(ConfigSensorPO vs : vehicleSensorVos){
            String sensorType = vs.getSensorType();
            String sensorId = vs.getSensorId();
            if(DictConstants.SENSOR_CAMERA.equals(sensorType)){
                //摄像头
                SensorBaseInfoVo sensorBaseInfoVo = simulationProjectMapper.selectSensorCamera(sensorId);
                setVehicleConfig(sensorBaseInfoVo,vs);
                sensorCameraList.add(sensorBaseInfoVo);
            }else if(DictConstants.SENSOR_OGT.equals(sensorType)){
                //完美传感器
                SensorBaseInfoVo sensorBaseInfoVo = simulationProjectMapper.selectSensorOgt(sensorId);
                setVehicleConfig(sensorBaseInfoVo,vs);
                sensorOgtList.add(sensorBaseInfoVo);
            }else if(DictConstants.SENSOR_LIDAR.equals(sensorType)){
                //激光雷达
                SensorBaseInfoVo sensorBaseInfoVo = simulationProjectMapper.selectSensorLidar(sensorId);
                setVehicleConfig(sensorBaseInfoVo,vs);
                sensorLidarList.add(sensorBaseInfoVo);
            }else if(DictConstants.SENSOR_RADAR.equals(sensorType)){
                //毫米波雷达
                SensorBaseInfoVo sensorBaseInfoVo = simulationProjectMapper.selectSensorRadar(sensorId);
                setVehicleConfig(sensorBaseInfoVo,vs);
                sensorRadarList.add(sensorBaseInfoVo);
            }else if(DictConstants.SENSOR_GPS.equals(sensorType)){
                //gps
                SensorBaseInfoVo sensorBaseInfoVo = simulationProjectMapper.selectSensorGps(sensorId);
                setVehicleConfig(sensorBaseInfoVo,vs);
                sensorGpsList.add(sensorBaseInfoVo);
            }
        }

        //任务信息
        ProjectTaskParam projectTaskParam = new ProjectTaskParam();
        projectTaskParam.setPId(po.getId());
        List<ProjectRunResultRatioNumVo> projectRunResultRatioNumVos = null;

        //任务运行状态统计
        List<ProjectRunStateNumVo> projectRunStateNumVos = null;
        Integer size = simulationProjectTaskMapper.selectProjectTaskNumByProjectId(projectTaskParam);
        if(size > 0){
            projectRunStateNumVos = simulationProjectTaskMapper.selectRunStateCount(po.getId());
            taskRunState(projectRunStateNumVos, size);


            projectRunResultRatioNumVos = simulationProjectTaskMapper.selectRunResultCount(po.getId());
            taskResultState(projectRunResultRatioNumVos, size);

        }

        projectDetailsVo.setProjectId(po.getProjectId());
        projectDetailsVo.setProjectName(po.getProjectName());
        projectDetailsVo.setProjectDescribe(po.getProjectDescribe());
        projectDetailsVo.setStartTime(getRqStr(po.getStartTime(),1));
        projectDetailsVo.setFinishTime(getRqStr(po.getFinishTime(),1));
        projectDetailsVo.setNowRunState(po.getNowRunState());
        projectDetailsVo.setNowRunStateName(getDictName(DictConstants.PROJECT_RUN_STATE, po.getNowRunState()));
//        projectDetailsVo.setEvaluationLevel(getDictName(DictConstants.EVALUATION_LEVEL, po.getEvaluationLevel()));
        projectDetailsVo.setEvaluationLevel(s);
        if(algorithmBaseInfoVo != null){
            projectDetailsVo.setAlgorithmName(algorithmBaseInfoVo.getAlgorithmName());
            projectDetailsVo.setAlgorithmDescribe(algorithmBaseInfoVo.getDescription());
        }
        if(vehicleBaseInfoVo != null){
            projectDetailsVo.setVehicleName(vehicleBaseInfoVo.getVehicleName());
            projectDetailsVo.setVehicleDescribe(vehicleBaseInfoVo.getDescription());
            projectDetailsVo.setVehicleTopView(vehicleBaseInfoVo.getVehicleTopView());

        }
        projectDetailsVo.setSensorCameraList(sensorCameraList);
        projectDetailsVo.setSensorOgtList(sensorOgtList);
        projectDetailsVo.setSensorLidarList(sensorLidarList);
        projectDetailsVo.setSensorRadarList(sensorRadarList);
        projectDetailsVo.setSensorGpsList(sensorGpsList);
        projectDetailsVo.setStateList(projectRunStateNumVos);
        projectDetailsVo.setResultList(projectRunResultRatioNumVos);

        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS, projectDetailsVo);
    }

    //运行状态统计
    private void taskRunState(List<ProjectRunStateNumVo> vos, Integer size){
        for(ProjectRunStateNumVo pv : vos){
            Integer num = pv.getNum();
            Double d = (double)num/size;
            d = saveTwoDecimalPlaces(d);
            pv.setRatio(d);
        }
        //补全没有的
        Map<String, String> dict = getDictByType(ProjectConstants.TASK_RUN_STATE);
        for(String key : dict.keySet()){
            boolean b = false;
            for(ProjectRunStateNumVo pv : vos){
                if(pv.getStateName().equals(dict.get(key))){
                    b = true;
                }
            }
            if(!b){
                ProjectRunStateNumVo vo = new ProjectRunStateNumVo();
                vo.setStateName(dict.get(key));
                vo.setNum(0);
                vo.setRatio(0D);
                vos.add(vo);
            }

        }
    }

    //运行结果统计
    private void taskResultState(List<ProjectRunResultRatioNumVo> vos,Integer size){
        //结果状态统计
        for(ProjectRunResultRatioNumVo rv : vos){
            Integer num = rv.getNum();
            Double d = (double)num/size;
            d = saveTwoDecimalPlaces(d);
            rv.setRatio(d);
        }
        //补全没有的
        Map<String, String> dict = getDictByType(ProjectConstants.TASK_RESULT_STATE);
        for(String key : dict.keySet()){
            boolean b = false;
            for(ProjectRunResultRatioNumVo pv : vos){
                if(pv.getResultName().equals(dict.get(key))){
                    b = true;
                }
            }
            if(!b){
                ProjectRunResultRatioNumVo vo = new ProjectRunResultRatioNumVo();
                vo.setResultName(dict.get(key));
                vo.setNum(0);
                vo.setRatio(0D);
                vos.add(vo);
            }
        }
    }

    private void setVehicleConfig(SensorBaseInfoVo vo, ConfigSensorPO vs){
        vo.setConfigId(vs.getConfigId());
        vo.setSensorId(vs.getSensorId());
        vo.setSensorType(vs.getSensorType());
        if(StringUtil.isNotEmpty(vs.getSensorX())){
            vo.setSensorX(saveTwoDecimalPlaces(vs.getSensorX().doubleValue()));
        }
        if(StringUtil.isNotEmpty(vs.getSensorY())){
            vo.setSensorY(saveTwoDecimalPlaces(vs.getSensorY().doubleValue()));
        }
        if(StringUtil.isNotEmpty(vs.getSensorZ())){
            vo.setSensorZ(saveTwoDecimalPlaces(vs.getSensorZ().doubleValue()));
        }
        if(StringUtil.isNotEmpty(vs.getSensorH())){
            vo.setSensorH(saveTwoDecimalPlaces(vs.getSensorH().doubleValue()));
        }
        if(StringUtil.isNotEmpty(vs.getSensorP())){
            vo.setSensorP(saveTwoDecimalPlaces(vs.getSensorP().doubleValue()));
        }
        if(StringUtil.isNotEmpty(vs.getSensorR())){
            vo.setSensorR(saveTwoDecimalPlaces(vs.getSensorR().doubleValue()));
        }
        if(StringUtil.isNotEmpty(vs.getSensorPort())){
            vo.setSensorPort(vs.getSensorPort());
        }


    }



    @Override
    public ResponseBodyVO selectProjectReportById(SimulationManualProjectParam param) {
        if(isEmpty(param.getId())){
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
        }
        //项目基本信息
        SimulationManualProjectPo po = simulationProjectMapper.selectProjectBaseById(param);
        if(po == null){
            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE, "没有获取到数据");
        }
        //算法配置
        AlgorithmPO algorithmBaseInfoVo = getAlgorithmInfo(po);

        ProjectReportVo projectReportVo = new ProjectReportVo();

        projectReportVo.setProjectId(po.getProjectId());
        projectReportVo.setProjectName(po.getProjectName());

        projectReportVo.setAlgorithmName(algorithmBaseInfoVo.getAlgorithmName());
//        Double algorithmScore = po.getAlgorithmScore();
//        projectReportVo.setAlgorithmScore(algorithmScore);
        projectReportVo.setAlgorithmDescribe(algorithmBaseInfoVo.getDescription());

//        String evaluationLevelReport = getEvaluationLevelReport(algorithmScore);
//        projectReportVo.setEvaluationLevel(evaluationLevelReport);


        String sceneNames = "";

        //算法测试得分表
        /*
        汇总测试得分计算方法：（每一项一级指标的测试得分*测试权重）累加
        得分率计算方法：每一项一级指标的所有场景得分不为0的数量/每一项一级指标的场景数量
        汇总得分率计算方法：每一项一级指标的所有场景得分不为0的数量_累加/每一项一级指标的场景数量_累加
         */
        List<ScenePackageSubListVO> scenePackageSubListVOS = simulationProjectMapper.selectSubSceneByPid(po.getScene());

        List<AlgorithmScoreVo> algorithmScoreVoList = new ArrayList();
        String evaluationLevelReport = "";
        if(!isEmpty(scenePackageSubListVOS)){
            StringBuffer stringBuffer = new StringBuffer("");

            //汇总数据初始化
            Integer totalSceneNum = 0;
//            Double totalWeight = 0D;
            Double totalScore = 0D;
            Double totalScoreRatio=0D;
            Integer totalSceneScoreNum = 0;

            for(ScenePackageSubListVO v : scenePackageSubListVOS){
                stringBuffer.append(v.getSublistName()+"、");
                AlgorithmScoreVo algorithmScoreVo = new AlgorithmScoreVo();
                algorithmScoreVo.setProjectName(v.getSublistName());
                Integer sceneNum = v.getSceneNum();
                totalSceneNum += sceneNum;
                algorithmScoreVo.setSceneNum(sceneNum);
                String weight = v.getWeight();
                Double weightDouble = Double.valueOf(weight);
                algorithmScoreVo.setWeight(weight);
                SimulationMptFirstTargetScorePo simulationMptFirstTargetScorePo = new SimulationMptFirstTargetScorePo();
                simulationMptFirstTargetScorePo.setPId(po.getId());
                simulationMptFirstTargetScorePo.setTarget(v.getSublistId());
                //单个二级指标得分
                SimulationMptFirstTargetScorePo simulationMptFirstTargetScorePo1 = simulationMptFirstTargetScoreMapper.selectFirstTargetScore(simulationMptFirstTargetScorePo);
                Double score = simulationMptFirstTargetScorePo1.getScore();
                totalScore += score*(weightDouble/100);
                algorithmScoreVo.setScore(saveTwoDecimalPlaces(score));
//                totalWeight +=aDouble;
//                Integer num = v.getSceneNum();
                //获取得分不为0的场景数量
                Integer scoreNum = getSetScoreNum(v.getSublistId(),po.getId());
                totalSceneScoreNum += scoreNum;
                algorithmScoreVo.setScoreRatio(saveTwoDecimalPlaces(Double.valueOf(scoreNum)/Double.valueOf(sceneNum)*100));
                algorithmScoreVoList.add(algorithmScoreVo);
            }
            //汇总
            AlgorithmScoreVo algorithmScoreVo = new AlgorithmScoreVo();
            algorithmScoreVo.setProjectName("汇总");
            algorithmScoreVo.setSceneNum(totalSceneNum);
//            algorithmScoreVo.setWeight(saveTwoDecimalPlaces(totalWeight).toString());
            //指标权重总和默认是100%
            algorithmScoreVo.setWeight("100");
            algorithmScoreVo.setScore(saveTwoDecimalPlaces(totalScore));
            totalScoreRatio = Double.valueOf(totalSceneScoreNum)/Double.valueOf(totalSceneNum)*100;
            algorithmScoreVo.setScoreRatio(saveTwoDecimalPlaces(totalScoreRatio));
            algorithmScoreVoList.add(algorithmScoreVo);

            projectReportVo.setScoreRatio(saveTwoDecimalPlaces(totalScoreRatio));

            projectReportVo.setAlgorithmScore(saveTwoDecimalPlaces(totalScore));
            evaluationLevelReport = getEvaluationLevelReport(saveTwoDecimalPlaces(totalScore));
            projectReportVo.setEvaluationLevel(evaluationLevelReport);

            sceneNames = stringBuffer.substring(0,stringBuffer.lastIndexOf("、"));
        }



        projectReportVo.setAlgorithmEvaluation(algorithmBaseInfoVo.getAlgorithmName()+"经测试获得"+evaluationLevelReport+"级评价，"+
                "("+sceneNames+")得分率达到了"+projectReportVo.getScoreRatio()+"。");

        projectReportVo.setAlgorithmScoreList(algorithmScoreVoList);
        projectReportVo.setEvaluationGrade(evaluationLevelReport);

        //算法测试评分细则、详细得分情况
        /*List<ScenePackageSubListVO> scenePackageSubListVOS1 = selectScenePackageSubListTreeAndSetScore(null, true, po.getScene(), po.getId(), 2);
        projectReportVo.setSceneScoreList(scenePackageSubListVOS1);*/

        //查询场景包所有数据
//        List<SublistScoreVo> pos = simulationProjectMapper.selectSubScore(po.getScene());

        //指标得分列表
        Map<String, Object> stringObjectMap = selectScenePackageSubListAndSetScore(po.getScene());
        projectReportVo.setSubListScoreLiTitle((List<Map>) stringObjectMap.get("cloums"));
        projectReportVo.setSubListScoreLi((List<SubScListVo>) stringObjectMap.get("result"));

        //场景得分列表
//        Map<String, Object> maps = selectSceneScore(pos, po.getId());
        Map<String, Object> maps = selectSceneScore(po.getScene(), po.getId());
        projectReportVo.setSceneScoreLiTitle((List<Map>) maps.get("cloums"));
        projectReportVo.setSceneScoreLi((List<SceneScListVo>) maps.get("result"));

        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS, projectReportVo);
    }

    private AlgorithmPO getAlgorithmInfo(SimulationManualProjectPo po){
        AlgorithmPO algorithmBaseInfoVo = null;
        String algorithmId = po.getAlgorithm();
        String algorithmType = po.getAlgorithmType();
        //第三方算法平台
        if("3".equals(algorithmType)){
            String sort = "algorithmId-desc";
            Integer page = 1;
            Integer size = 1;//全部

            String urlParam = "&algorithmId"+algorithmId+"&page=" + page+ "&size=" + size+ "&sort=" + sort;
            List<DropDownVo> otherAlgorithmInfo = getOtherAlgorithmInfo(urlParam);
            if(StringUtil.isNotEmpty(otherAlgorithmInfo)){
                DropDownVo dropDownVo = otherAlgorithmInfo.get(0);
                algorithmBaseInfoVo = new AlgorithmPO();
                algorithmBaseInfoVo.setAlgorithmName(dropDownVo.getName());
                algorithmBaseInfoVo.setDescription(dropDownVo.getDescription());
            }

        }else{
            AlgorithmPO algorithmPO = new AlgorithmPO();
            algorithmPO.setId(algorithmId);
            algorithmBaseInfoVo = simulationProjectMapper.selectAlgorithmBaseInfoById(algorithmPO).get(0);
        }
        return algorithmBaseInfoVo;
    }


    @Override
    public ResponseBodyVO selectDropDownByType(SimulationManualProjectParam param) {
        List<DropDownTypeVo> result = new ArrayList<>();
        if(isEmpty(param.getDropDownType())){
            //获取全部（算法，车辆，场景）
            setAlgorithmDropDown(result,param.getAlgorithmType());
            setVehicleDropDown(result);
            setScenePackageDropDown(result);
        }else if("1".equals(param.getDropDownType())){
            setAlgorithmDropDown(result,param.getAlgorithmType());
        }else if("2".equals(param.getDropDownType())){
            setVehicleDropDown(result);
        }else if("3".equals(param.getDropDownType())){
            setScenePackageDropDown(result);
        }
        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS, result);
    }

    @Override
    public ResponseBodyVO selectProjectTaskById(SimulationManualProjectParam param) {
        String id = param.getId();
        String taskId = param.getTaskId();
        if(isEmpty(id) || isEmpty(taskId)){
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
        }

        ProjectTaskDetailsVo resultVo = new ProjectTaskDetailsVo();

        //基本信息
        ProjectTaskParam projectTaskParam = new ProjectTaskParam();
        projectTaskParam.setPId(id);
        projectTaskParam.setId(taskId);
        List<ManualProjectTaskPo> pos = simulationProjectTaskMapper.selectprojectTaskByQuery(projectTaskParam);
        if(isEmpty(pos)){
            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"没有获取到任务信息");
        }
        ManualProjectTaskPo po = pos.get(0);
        resultVo.setTaskId(po.getId());
        String sceneId = po.getSceneId();
        String sceneType = po.getSceneType();
        SceneBaseInfoVo sceneBaseInfoVo = getsceneNameAndOther(sceneId, sceneType);
        resultVo.setSceneName(sceneBaseInfoVo.getCommonSceneName());
        resultVo.setRunStartTime(getRqStr(po.getRunStartTime(),1));
        resultVo.setRunEndTime(getRqStr(po.getRunEndTime(),1));
        resultVo.setRunState(po.getRunState());
        resultVo.setFinishState(po.getRunResult());
        resultVo.setSceneDescribe("");


        ///TODO 1.获取仿真结果文件
        InputStream fileInputStream = null;
        InputStreamReader inputStreamReader = null;
        BufferedReader bufferedReader = null;
        try {
            //2.解析仿真文件
            //time
            List<Double> time_list = new ArrayList<>();
            //velocity
            List<Double> velocity_list = new ArrayList<>();
            List<Double> lateral_velocity_list = new ArrayList<>();//横  时间
            List<Double> longitudinal_velocity_list = new ArrayList<>();//纵 速度
            //acceleration
            List<Double> acceleration_list = new ArrayList<>();
            List<Double> lateral_acceleration = new ArrayList<>();//纵向加速度
            List<Double> longitudinal_acceleration_list = new ArrayList<>();//纵向加速度
            //lane_offset
            List<Double> lane_offset_list = new ArrayList<>();
            ///TODO brake
            List<Double> brake_list = new ArrayList<>();
            //steeting_wheel
            List<Double> steeting_wheel_list = new ArrayList<>();
            ///TODO throttle
            List<Double> throttle_list = new ArrayList<>();


            List<Double> yawrate_list = new ArrayList<>();//摆角速度

            File file = new File("E:\\仿真云平台\\任务详情界面数据\\Ego(1).csv");
            fileInputStream = new FileInputStream(file);
            inputStreamReader = new InputStreamReader(fileInputStream,"utf-8");

/*            MinioParameter minioParameter = new MinioParameter();
//            minioParameter.setObjectName(po.getRunResultFilePath()+"/ego.csv");
            minioParameter.setObjectName("test/ego.csv");
            Response download = fileDownService.download(minioParameter);
            Response.Body body = download.body();
            InputStream inputStream = body.asInputStream();
            InputStreamReader inputStreamReader = new InputStreamReader(inputStream,"utf-8");*/

            bufferedReader = new BufferedReader(inputStreamReader);
            String line;
            int lineIndex = 0;
            while((line = bufferedReader.readLine()) != null){
                lineIndex++;
                if(lineIndex == 1){
                    continue;
                }
                String[] split = line.split(",");

                try {
                    time_list.add(Double.valueOf(split[1]));
                }catch (NumberFormatException e){
                    time_list.add(0D);
                }


                try {
                    lateral_velocity_list.add(Double.valueOf(split[6]));
                }catch (NumberFormatException e){
                    lateral_velocity_list.add(0D);
                }

                try {
                    longitudinal_velocity_list.add(Double.valueOf(split[7]));
                }catch (NumberFormatException e){
                    longitudinal_velocity_list.add(0D);
                }


                try {
                    lateral_acceleration.add(Double.valueOf(split[8]));
                }catch (NumberFormatException e){
                    lateral_acceleration.add(0D);
                }

                try {
                    longitudinal_acceleration_list.add(Double.valueOf(split[9]));
                }catch (NumberFormatException e){
                    longitudinal_acceleration_list.add(0D);
                }

                try {
                    steeting_wheel_list.add(Double.valueOf(split[12]));//steering_angle
                }catch (NumberFormatException e){
                    steeting_wheel_list.add(0D);
                }

                try {
                    yawrate_list.add(Double.valueOf(split[13]));
                }catch (NumberFormatException e){
                    yawrate_list.add(0D);
                }

                try {
                    lane_offset_list.add(Double.valueOf(split[27]));//lane_center_offset
                }catch (NumberFormatException e){
                    lane_offset_list.add(0D);
                }

//                brake_list.add(Double.valueOf(split[7])); ///TODO 没有
//                throttle_list.add(Double.valueOf(split[7]));///TODO 没有

            }
            //里程
            Double lc = 0D;
            for(int i=0; i<time_list.size(); i++){
                 if(i == 0){
                     continue;
                 }
                lc += (time_list.get(i)-time_list.get(i-1))*longitudinal_velocity_list.get(i-1);

            }

            //平均速度
            Double pjsd = lc/time_list.get(time_list.size()-1);

            //最大速度
            Double zdsd = Collections.max(longitudinal_velocity_list);

            //最小速度
            Double zxsd = Collections.min(longitudinal_velocity_list);

            //最大加速度
            Double zdjiasd = Collections.max(longitudinal_acceleration_list);

            //最大减速度
            Double zdjiansd = Collections.min(longitudinal_acceleration_list);

            //最大摆角速度
            Double zdbjsd = Collections.max(yawrate_list);

            //自车速度方差
            Double zcsdfc = 0D;
            for(Double d :longitudinal_velocity_list){
                zcsdfc +=Math.pow(d-pjsd,2);
            }
            zcsdfc = zcsdfc/longitudinal_velocity_list.size();

            //自车横摆角速度均方根
            Double zchbjsdjfg = 0D;
            for(Double d :yawrate_list){
                zchbjsdjfg += Math.pow(d-pjsd,2);
            }
            zchbjsdjfg = Math.sqrt(zchbjsdjfg/yawrate_list.size());

            for(int i=0; i<lateral_acceleration.size(); i++){
                Double aDouble = lateral_acceleration.get(i);
                Double aDouble1 = longitudinal_acceleration_list.get(i);
                Double d = Math.pow(aDouble,2)+Math.pow(aDouble1,2);
                acceleration_list.add(Math.sqrt(d));
            }

            for(int i=0; i<lateral_velocity_list.size(); i++){
                Double aDouble = lateral_velocity_list.get(i);
                Double aDouble1 = longitudinal_velocity_list.get(i);
                Double d = Math.pow(aDouble,2)+Math.pow(aDouble1,2);
                velocity_list.add(d);
            }

            //任务详情信息
            resultVo.setMileage(lc.toString());
            resultVo.setAverageVelocity(pjsd.toString());
            resultVo.setMaximunSpeed(zdsd.toString());
            resultVo.setMinimunVelocity(zxsd.toString());
            resultVo.setMaximumAcceleration(zdjiasd.toString());
            resultVo.setMaximumDeceleration(zdjiansd.toString());
            resultVo.setMaximumSwingSpeed(zdbjsd.toString());
            resultVo.setSpeedVariance(zcsdfc);
//            resultVo.setSpeedComfortLevel(po.getSpeedComfortLevel());

            resultVo.setSwingSpeedMeanSquareRoot(zchbjsdjfg);
//            resultVo.setSwingComfortLevel(po.getSwingComfortLevel());

            HashMap<String, List> CurveData = new HashMap<>();
            CurveData.put("time",time_list);
            CurveData.put("acceleration",acceleration_list);
            CurveData.put("lane_offset",lane_offset_list);
            CurveData.put("brake",brake_list);
            CurveData.put("steeting",steeting_wheel_list);
            CurveData.put("throttle",throttle_list);
            CurveData.put("yaw_rate",yawrate_list);
            CurveData.put("velocity",velocity_list);
            resultVo.setCurveData(CurveData);


        } catch (Exception e){
            e.printStackTrace();

        }finally {
            try {
                if(fileInputStream != null){
                    fileInputStream.close();
                }
                if(inputStreamReader != null){
                    inputStreamReader.close();
                }
                if(bufferedReader != null){
                    bufferedReader.close();
                }
            }catch (IOException e){
                e.printStackTrace();

            }

        }


        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,resultVo);
    }

    @Override
    public ResponseBodyVO selectMaxParallelism() {
        String currentUserId = AuthUtil.getCurrentUserId();
        SystemUserVo systemUserVo = simulationProjectMapper.selectUserById(currentUserId);
        if(systemUserVo == null){
            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"没有获取到用户信息");
        }

        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,null,systemUserVo.getCpuAvailableNumber());
    }

    @Override
    public String getDictName(String type, String code) {
        Map<String, String> stringStringMap = getDictByType(type);
        if(stringStringMap != null && stringStringMap.size() > 0){
            return stringStringMap.get(code);
        }
        return "";
    }

    private Map<String, String> getDictByType(String type){
        DictParam dictParam = new DictParam();
        dictParam.setDictTypes(type);
        Map<String, Map<String, String>> dictMapsByTypes = dictService.getDictMapsByTypes(dictParam);
        Map<String, String> stringStringMap = dictMapsByTypes.get(type);
        if(stringStringMap != null && stringStringMap.size() > 0){
            return stringStringMap;
        }
        return null;

    }

    @Override
    public ResponseBodyVO analysisVehicleCoord() throws Exception {

        analysisCsvFile("E:\\仿真云平台\\三维还原视频说明\\360ogt.csv",new int[]{6,7,9});

        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,null);

    }

    @Override
    public ResponseBodyVO selectProjectTaskList(SimulationManualProjectParam param) {
        if(isEmpty(param.getId())){
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
        }
        //查询任务信息
        ProjectTaskParam projectTaskParam = new ProjectTaskParam();
        projectTaskParam.setPId(param.getId());
        setPage(param.getCurrentPage()==null?1:param.getCurrentPage(), param.getPageSize()==null?10:param.getPageSize());
        List<ManualProjectTaskVo> manualProjectTaskVos = simulationProjectTaskMapper.selectProjectTaskByProjectId(projectTaskParam);

        PageInfo<ManualProjectTaskVo> pageInfo = new PageInfo(manualProjectTaskVos);

        if(pageInfo.getList() != null){
            for(ManualProjectTaskVo task : pageInfo.getList()){
                task.setRunStartTimeFmt(getRqStr(task.getRunStartTime(), 2));
                task.setRunEndTimeFmt(getRqStr(task.getRunEndTime(), 2));
                setUpSceneInfo(task);

            }
        }
        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,pageInfo);
    }

    private void analysisCsvFile(String filePath, int[] analysisFields) throws Exception {

        File file = new File(filePath);
        InputStream fileInputStream = new FileInputStream(file);
        InputStreamReader inputStreamReader = new InputStreamReader(fileInputStream,"utf-8");
        BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
        String line;
        while((line = bufferedReader.readLine()) != null){
            String[] split = line.split(",");
            for(int fs : analysisFields){
                System.out.println(split[fs]);
            }

        }
    }

    /**
     * 根据场景id和场景类型获取场景名
     * @return
     */
    private SceneBaseInfoVo getsceneNameAndOther(String sceneId, String sceneType){
        SceneBaseInfoVo resultVo = new SceneBaseInfoVo();
        if(DictConstants.SCENE_NATURAL.equals(sceneType)){
            SceneBaseInfoVo sceneBaseInfoVo = simulationProjectMapper.selectSceneNatural(sceneId);
            if(sceneBaseInfoVo != null){
                resultVo.setCommonSceneName(sceneBaseInfoVo.getNaturalName());
            }
        }else if(DictConstants.SCENE_STANDARD.equals(sceneType)){
            SceneBaseInfoVo sceneBaseInfoVo = simulationProjectMapper.selectSceneStandardsRegulations(sceneId);
            if(sceneBaseInfoVo != null){
                resultVo.setCommonSceneName(sceneBaseInfoVo.getSceneName());
            }
        }else if(DictConstants.SCENE_ACCIDENT.equals(sceneType)){
            SceneBaseInfoVo sceneBaseInfoVo = simulationProjectMapper.selectSceneAccidentById(sceneId);
            if(sceneBaseInfoVo != null){
                resultVo.setCommonSceneName(sceneBaseInfoVo.getSceneName());
            }
        }else if(DictConstants.SCENE_GENERAL.equals(sceneType)){
            ///TODO 泛化场景暂不支持

        }

        return resultVo;

    }

    private void setAlgorithmDropDown(List<DropDownTypeVo> result, String algorithmType){

        List<DropDownVo> algorithmList = new ArrayList<>();

        if(ProjectConstants.SY_ALGORITHM_TYPE.equals(algorithmType)){
            //第三方算法平台获取（索为）
            String sort = "algorithmId-desc";
            Integer page = 1;
            Integer size = 999;//全部

            String urlParam = "&page=" + page+ "&size=" + size+ "&sort=" + sort;
            algorithmList = getOtherAlgorithmInfo(urlParam);

        }else{
            AlgorithmPO algorithmPO = new AlgorithmPO();
            algorithmPO.setCreateUserId(AuthUtil.getCurrentUserId());
            List<AlgorithmPO> algorithmBaseInfoVo = simulationProjectMapper.selectAlgorithmBaseInfoById(algorithmPO);

            for(AlgorithmPO v : algorithmBaseInfoVo){
                DropDownVo dropDownVo = new DropDownVo();
                dropDownVo.setId(v.getId());
                dropDownVo.setName(v.getAlgorithmName());
                dropDownVo.setDescription(v.getDescription());
                algorithmList.add(dropDownVo);
            }

        }
        DropDownTypeVo algorithmDropDown = new DropDownTypeVo();
        algorithmDropDown.setDropDownList(algorithmList);
        algorithmDropDown.setType("1");
        result.add(algorithmDropDown);

    }

    /**
     * 获取第三方算法信息
     * @return
     */
    private List<DropDownVo> getOtherAlgorithmInfo(String query){

        List<DropDownVo> algorithmList = new ArrayList<>();

        ResponseBodyVO algorithmBody = algoPlatformService.getAlgorithmList(query);

        //解析数据
        Map jsonMap = JsonUtil.jsonToMap((String) algorithmBody.getInfo());
        Map<String, Object> dataMap = (Map<String, Object>) jsonMap.get("data");
        List<Map<String, String>> contentList = (List<Map<String, String>>) dataMap.get("content");
        Integer totalElements = (Integer) dataMap.get("totalElements");
        if(totalElements > 0){
            for (Map<String, String> content: contentList) {
                DropDownVo dropDownVo = new DropDownVo();
                dropDownVo.setId(content.get("algorithmId"));
                dropDownVo.setName(content.get("algorithmName"));
                dropDownVo.setDescription(content.get("description"));
                algorithmList.add(dropDownVo);
            }
        }
        return algorithmList;


    }

    private void setVehicleDropDown(List<DropDownTypeVo> result){
        ConfigPO configPO = new ConfigPO();
        configPO.setCreateUserId(AuthUtil.getCurrentUserId());
        List<ConfigPO> vehicleBaseInfoVo = simulationProjectMapper.selectConfigVehicle(configPO);
        List<DropDownVo> vehicleList = new ArrayList<>();
        for(ConfigPO v : vehicleBaseInfoVo){
            DropDownVo dropDownVo = new DropDownVo();
            dropDownVo.setId(v.getId());
            dropDownVo.setName(v.getConfigName());
            dropDownVo.setDescription(v.getDescription());
            //获取传感器信息
            List<ConfigSensorPO> configSensorVos = simulationProjectMapper.selectConfigSensor(v.getId());
            String sensor = "";
            if(!isEmpty(configSensorVos) && configSensorVos.get(0) != null){
                StringBuffer stringBuffer = new StringBuffer();
                for(ConfigSensorPO cv : configSensorVos){
                    stringBuffer.append(cv.getSensorType()+",");
                }
                sensor = stringBuffer.substring(0,stringBuffer.lastIndexOf(","));

            }
            dropDownVo.setSensor(sensor);

            vehicleList.add(dropDownVo);
        }

        DropDownTypeVo vehicleDropDown = new DropDownTypeVo();
        vehicleDropDown.setDropDownList(vehicleList);
        vehicleDropDown.setType("2");
        result.add(vehicleDropDown);
    }
    private void setScenePackageDropDown(List<DropDownTypeVo> result){
        ScenePackagePO scenePackagePO = new ScenePackagePO();
        scenePackagePO.setCreateUserId(AuthUtil.getCurrentUserId());
        List<ScenePackagePO> scenePackageBaseVo = simulationProjectMapper.selectScenePackageBaseById(scenePackagePO);
        List<DropDownVo> scenePackageList = new ArrayList<>();
        for(ScenePackagePO v : scenePackageBaseVo){
            DropDownVo dropDownVo = new DropDownVo();
            dropDownVo.setId(v.getPackageId());
            dropDownVo.setName(v.getPackageName());
            dropDownVo.setSceneNum(String.valueOf(v.getSceneNum()));
            scenePackageList.add(dropDownVo);
        }
        DropDownTypeVo scenePackageDropDown = new DropDownTypeVo();
        scenePackageDropDown.setDropDownList(scenePackageList);
        scenePackageDropDown.setType("3");
        result.add(scenePackageDropDown);
    }


    private SimulationManualProjectPo convertParamToPo(SimulationManualProjectParam param){
        SimulationManualProjectPo po = new SimulationManualProjectPo();
        po.setId(param.getId());
        po.setProjectName(param.getProjectName());
        po.setProjectDescribe(param.getProjectDescribe());
        po.setAlgorithm(param.getAlgorithm());
        po.setVehicle(param.getVehicle());
        po.setScene(param.getScene());
        //po.setOperationCycle(param.getOperationCycle());
        po.setMaxSimulationTime(param.getMaxSimulationTime());
        po.setParallelism(param.getParallelism());
        //po.setRuleView(param.getRuleView());
        po.setIsChoiceGpu(param.getIsChoiceGpu());
        po.setNowRunState(param.getNowRunState());
        po.setAlgorithmType(param.getAlgorithmType());
        return po;

    }

    private SimulationManualProjectVo convertPoToVo(SimulationManualProjectVo vo){
        Date createTime = vo.getCreateTime();
        vo.setCreateTimeFmt(getRqStr(createTime, 2));
        String algorithm = vo.getAlgorithm();

        AlgorithmPO algorithmPO = new AlgorithmPO();
        algorithmPO.setId(algorithm);
        List<AlgorithmPO> algorithmBaseInfoVos = simulationProjectMapper.selectAlgorithmBaseInfoById(algorithmPO);
        if(!isEmpty(algorithmBaseInfoVos)){
            AlgorithmPO algorithmBaseInfoVo = algorithmBaseInfoVos.get(0);
            vo.setAlgorithm(algorithmBaseInfoVo.getAlgorithmName());
        }
        vo.setNowRunStateDict(getDictName(DictConstants.PROJECT_RUN_STATE, vo.getNowRunState()));
        vo.setEvaluationLevelDict(getDictName(DictConstants.EVALUATION_LEVEL,vo.getEvaluationLevel()));
        return vo;
    }

    private void convertPoToVo (SimulationManualProjectPo po, SimulationManualProjectSingleVo vo){
        vo.setId(po.getId());
        vo.setProjectName(po.getProjectName());
        vo.setProjectDescribe(po.getProjectDescribe());
        vo.setAlgorithm(po.getAlgorithm());
        vo.setAlgorithmType(po.getAlgorithmType());
        vo.setVehicle(po.getVehicle());
        vo.setScene(po.getScene());
        vo.setMaxSimulationTime(po.getMaxSimulationTime());
        vo.setParallelism(po.getParallelism());
        vo.setIsChoiceGpu(po.getIsChoiceGpu());

    }

    private void createProjectId(SimulationManualProjectPo po){
        Integer nowRq = getRq(null,0);
        po.setProjectDate(nowRq);
        SimulationManualProjectPo po1 = simulationProjectMapper.selectLastProjectId(nowRq);
        if(po1 == null){
            //生成新id
            po.setProjectNum(1);
        }else{
            po.setProjectNum(po1.getProjectNum()+1);
        }
        po.setProjectId(po.getProjectDate()+"-"+po.getProjectNum());

    }

    private Integer getRq(Date date, int index){
        return  TimeUtil.getRq(date,index);
    }

    private String getRqStr(Date date, int index){
        SimpleDateFormat sdf = new SimpleDateFormat(dateFmtArr[index]);
        if(date == null){
            return "";
        }
        return sdf.format(date);

    }

    private Date getDate(String dateFmt, int index){
        SimpleDateFormat sdf = new SimpleDateFormat(dateFmtArr[index]);
        try {
            return sdf.parse(dateFmt);
        } catch (ParseException e) {
            return null;
        }
    }

    private boolean isEmpty(String value){
        if(value == null){
            return true;
        }
        value = value.trim();
        if(value.length() == 0){
            return true;
        }
        return false;
    }
    private boolean isEmpty(List list){
        if(list == null || list.size() <= 0){
            return true;
        }
        return false;
    }

    private void setPage(Integer pageNum, Integer pageSize){
        PageVO pageVO = new PageVO();
        pageVO.setCurrentPage(pageNum);
        pageVO.setPageSize(pageSize);
        PageUtil.setPageInfo(pageVO);
    }

    private Double saveTwoDecimalPlaces(Double d){
        if(d == null){
            return null;
        }
        return new BigDecimal(d).setScale(2, RoundingMode.UP).doubleValue();
    }

    /**
     * 设置场景基本字段
     */
    private void setUpSceneInfo(ManualProjectTaskVo vo){
        String sceneType = vo.getSceneType();
        String sceneId = vo.getSceneId();
        SceneBaseInfoVo sceneBaseInfoVo;
        if(DictConstants.SCENE_NATURAL.equals(sceneType)){
            //自然驾驶
            sceneBaseInfoVo = simulationProjectMapper.selectSceneNatural(sceneId);
            if(sceneBaseInfoVo != null){
                vo.setSceneName(sceneBaseInfoVo.getNaturalName());
            }
        }else if(DictConstants.SCENE_STANDARD.equals(sceneType)){
            //标准法规
            sceneBaseInfoVo = simulationProjectMapper.selectSceneStandardsRegulations(sceneId);
            if(sceneBaseInfoVo != null){
                vo.setSceneName(sceneBaseInfoVo.getSceneName());
            }
        }else if(DictConstants.SCENE_ACCIDENT.equals(sceneType)){
            //交通事故
            sceneBaseInfoVo = simulationProjectMapper.selectSceneAccidentById(sceneId);
            if(sceneBaseInfoVo != null){
                vo.setSceneName(sceneBaseInfoVo.getSceneName());
            }
        }else if(DictConstants.SCENE_GENERAL.equals(sceneType)){
            /// TODO 泛化场景暂不支持

        }
    }

    private ManualProjectTaskVo convertManualProjectTaskPoToVo(ManualProjectTaskPo po){
        ManualProjectTaskVo manualProjectTaskVo = new ManualProjectTaskVo();
        manualProjectTaskVo.setId(po.getId());
        manualProjectTaskVo.setPId(po.getPId());
        manualProjectTaskVo.setSceneId(po.getSceneId());
//        manualProjectTaskVo.setSceneName(po.getSceneName());
        manualProjectTaskVo.setSceneType(po.getSceneType());
        manualProjectTaskVo.setRunStartTimeFmt(getRqStr(po.getRunStartTime(), 2));
        manualProjectTaskVo.setRunEndTimeFmt(getRqStr(po.getRunEndTime(), 2));
        manualProjectTaskVo.setRunState(po.getRunState());
        manualProjectTaskVo.setRunResult(po.getRunResult());
        return manualProjectTaskVo;
    }


    /**
     * 级联获取场景指标得分数据
     * @param parentVoList 首次传null
     * @param isRoot 首次传true
     * @param scenePackageId 场景包id
     * @param pId 项目表id，用于统计得分使用
     * @param level 指标级别
     * @return
     */
    private List<ScenePackageSubListVO> selectScenePackageSubListTreeAndSetScore(List<ScenePackageSubListVO> parentVoList,
                                                                      boolean isRoot,String scenePackageId, String pId, Integer level){

        if(isRoot){
            //查找一级节点
            parentVoList = simulationProjectMapper.selectSubSceneByPid(scenePackageId);
            if(!isEmpty(parentVoList)){
                selectScenePackageSubListTreeAndSetScore(parentVoList, false, null,pId,level);
            }

        }else{

            //获取子节点集合
            for(ScenePackageSubListVO pvo : parentVoList){

                pvo.setLevel(level);

                //二级指标获取总分
                if(level == 2){
                    setFirstTargetScore(pvo,pId);
                }

                List<ScenePackageSubListVO> cvoList = simulationProjectMapper.selectSubSceneByPid(pvo.getSublistId());
                if(!isEmpty(cvoList)){
                    //存入父节点集合中
                    pvo.setChildScenePackageSubListVOList(cvoList);
                    //继续查找下一节点
                    selectScenePackageSubListTreeAndSetScore(cvoList,false, null, pId, level+1);
                }else{

                    //没有子节点；最后一级，获取指标得分,指标得分说明,和指标下场景得分
                    setScore(pvo,pId);
                }

            }
        }


        return parentVoList;
    }


    private Map<String, Object> selectSceneScore(String scenePackageId, String projectId) {

        //查询场景包所有数据
        List<SublistScoreVo> pos = simulationProjectMapper.selectSubScore(scenePackageId);

        HashMap<String, Object> hashMap = new HashMap<>();

        //1.获取项目下所有场景得分
        ProjectTaskParam param = new ProjectTaskParam();
        param.setPId(projectId);
        List<ManualProjectTaskVo> manualProjectTaskVos = simulationProjectTaskMapper.selectProjectTaskByProjectId(param);

        List<SublistScoreVo> lastSubList = new ArrayList<>();
        //2.拼接场景得分信息
        for(SublistScoreVo po : pos){
            if(ObjectUtil.isNotNull(po.getPackageAndRules())){

                //获取场景得分信息
                for(ManualProjectTaskVo sc : manualProjectTaskVos)
                    if (sc.getLastTargerId().equals(po.getId())) {
                        SublistScoreVo sublistScoreVo = new SublistScoreVo();
                        BeanUtils.copyProperties(po, sublistScoreVo);

                        sublistScoreVo.setSceneScore(sc.getScore()); //得分
                        sublistScoreVo.setTargetEvaluate(sc.getTargetEvaluate());//指标评价
                        sublistScoreVo.setReturnSceneId(sc.getReturnSceneId());//显示的场景id
                        String sceneType = sc.getSceneType();
                        if(DictConstants.SCENE_NATURAL.equals(sceneType)){
                            //自然驾驶
                            sublistScoreVo.setSceneType("自然驾驶");
                        }else if(DictConstants.SCENE_STANDARD.equals(sceneType)){
                            //标准法规
                            sublistScoreVo.setSceneType("标准法规");
                        }else if(DictConstants.SCENE_ACCIDENT.equals(sceneType)){
                            //交通事故
                            sublistScoreVo.setSceneType("交通事故");
                        }else if(DictConstants.SCENE_GENERAL.equals(sceneType)){
                            /// TODO 泛化场景暂不支持

                        }

                        lastSubList.add(sublistScoreVo);
                    }

            }
        }

        int size = 0;
        List<List<String>> result = new ArrayList<>();
        //末级指标
        for(SublistScoreVo sp : lastSubList){
            setParentSub(sp, null, pos);
            String sublistName = sp.getSublistName();
            sublistName += ","+sp.getReturnSceneId()+","+sp.getSceneType()+","+sp.getSceneScore()+","+sp.getTargetEvaluate()+","+sp.getScoreExplain();

            String[] split = sublistName.split(",");

            List<String> strings = new LinkedList<>();
            strings.addAll(Arrays.asList(split));
            if(size<strings.size()){
                size = strings.size();
            }
            result.add(strings);

        }

        List<SceneScListVo> objects = new ArrayList<>();
        //结果补全
        for(List<String> list : result){
            int start = list.size() - 5;
            SceneScListVo sceneScListVo = new SceneScListVo();
            for(int i=0;i<start;i++){
                if(0 == i){
                    sceneScListVo.setSublistName1(list.get(i));
                }else if(1 == i){
                    sceneScListVo.setSublistName2(list.get(i));
                }else if(2 == i){
                    sceneScListVo.setSublistName3(list.get(i));
                }else if(3 == i){
                    sceneScListVo.setSublistName4(list.get(i));
                }else if(4 == i){
                    sceneScListVo.setSublistName5(list.get(i));
                }else if(5 == i){
                    sceneScListVo.setSublistName6(list.get(i));
                }

            }

            sceneScListVo.setSceneId(list.get(start));
            sceneScListVo.setSceneIdType(list.get(start+1));
            sceneScListVo.setSceneScore(list.get(start+2));
            sceneScListVo.setTargetEvaluate(list.get(start+3));
            sceneScListVo.setScoreExplain(list.get(start+4));

            objects.add(sceneScListVo);
        }

        List<Map<String, String>> cloums = new ArrayList<>();
        for(int i=0; i<size-5;i++){
            setCloums(i,cloums);
        }

        Map<String, String> map1 = new HashMap<>();
        map1.put("label","场景ID");
        map1.put("prop","sceneId");
        cloums.add(map1);

        Map<String, String> map2 = new HashMap<>();
        map2.put("label","场景类型");
        map2.put("prop","sceneIdType");
        cloums.add(map2);

        Map<String, String> map3 = new HashMap<>();
        map3.put("label","测试得分");
        map3.put("prop","sceneScore");
        cloums.add(map3);

        Map<String, String> map4 = new HashMap<>();
        map4.put("label","指标评价");
        map4.put("prop","targetEvaluate");
        cloums.add(map4);

        Map<String, String> map5 = new HashMap<>();
        map5.put("label","得分说明");
        map5.put("prop","scoreExplain");
        cloums.add(map5);

        hashMap.put("cloums",cloums);
        hashMap.put("result",objects);

        return hashMap;


    }

    /**
     *
     * @param po 末级指标
     * @param parentPo 上级指标
     * @param pos 所有指标
     */
    private void setParentSub(SublistScoreVo po, SublistScoreVo parentPo, List<SublistScoreVo> pos) {
        if(parentPo == null){
            for(SublistScoreVo p : pos){
                setSc(po,p,po,pos);
            }

        }else{
            for(SublistScoreVo p : pos){
                setSc(parentPo,p,po,pos);
            }
        }
    }

    /**
     *
     * @param p1 上级指标
     * @param p2 比对指标
     * @param p3 末级指标
     * @param pos 所有指标
     */
    private void setSc(SublistScoreVo p1, SublistScoreVo p2, SublistScoreVo p3, List<SublistScoreVo> pos){
        //上级指标
        if(p1.getParentId().equals(p2.getId())){

            //末级指标拼接指标名
            p3.setSublistName(p2.getSublistName()+","+p3.getSublistName());
            //查找上级指标，继续拼接
            setParentSub(p3,p2, pos);
        }


    }




    private Map<String,Object> selectScenePackageSubListAndSetScore(String scenePackageId){

        //查询场景包所有数据
        List<SublistScoreVo> pos = simulationProjectMapper.selectSubScore(scenePackageId);

        HashMap<String, Object> hashMap = new HashMap<>();

        int size = 0;
        List<List<String>> result = new ArrayList<>();
        //递归指标名称得分
        for(SublistScoreVo po : pos){
            if(ObjectUtil.isNotNull(po.getPackageAndRules())){
                //末级指标
                setParent(po,null,pos,scenePackageId);

                String sublistName = po.getSublistName();
                String firstScore = po.getFirScore();//first得分
                String LastScore = po.getLasScore();//last得分
                String sceneNum = po.getSceneNum();//场景数量
                String notStandardSceneNum = po.getNotStandardSceneNum();//未达标场景数量

                if(ObjectUtil.isNotNull(sublistName)){
                    sublistName += ","+sceneNum+","+notStandardSceneNum+","+LastScore+","+firstScore;
                    String[] split = sublistName.split(",");


                    List<String> strings = new LinkedList<>();
                    strings.addAll(Arrays.asList(split));
                    if(size<strings.size()){
                        size = strings.size();
                    }
                    result.add(strings);
                }

            }

        }


        List<SubScListVo> objects = new ArrayList<>();
        //结果补全
        for(List<String> list : result){
            int start = list.size() - 4;
            SubScListVo subScListVo = new SubScListVo();
            for(int i=0;i<start;i++){
                if(0 == i){
                    subScListVo.setSublistName1(list.get(i));
                }else if(1 == i){
                    subScListVo.setSublistName2(list.get(i));
                }else if(2 == i){
                    subScListVo.setSublistName3(list.get(i));
                }else if(3 == i){
                    subScListVo.setSublistName4(list.get(i));
                }else if(4 == i){
                    subScListVo.setSublistName5(list.get(i));
                }else if(5 == i){
                    subScListVo.setSublistName6(list.get(i));
                }

            }
            subScListVo.setSceneNum(list.get(start));
            subScListVo.setNotStandardSceneNum(list.get(start+1));
            subScListVo.setLastScore(list.get(start+2));
            subScListVo.setFirstScore(list.get(start+3));

            objects.add(subScListVo);


        }

        List<Map<String, String>> cloums = new ArrayList<>();
        for(int i=0; i<size-4;i++){
            setCloums(i,cloums);
        }
        Map<String, String> map1 = new HashMap<>();
        map1.put("label","场景数量");
        map1.put("prop","sceneNum");
        cloums.add(map1);

        Map<String, String> map2 = new HashMap<>();
        map2.put("label","未达标场景数量");
        map2.put("prop","notStandardSceneNum");
        cloums.add(map2);

        Map<String, String> map3 = new HashMap<>();
        map3.put("label","得分");
        map3.put("prop","lastScore");
        cloums.add(map3);

        Map<String, String> map4 = new HashMap<>();
        map4.put("label","总分");
        map4.put("prop","firstScore");
        cloums.add(map4);

        hashMap.put("cloums",cloums);
        hashMap.put("result",objects);


        return hashMap;


    }

    /**
     *
     * @param po 末级指标
     * @param pos 场景包下所有指标
     * @param rootId 场景包id，用于判断是否根指标
     */
    private void setParent(SublistScoreVo po, List<SublistScoreVo> pos, String rootId) {
        for(SublistScoreVo p : pos){

            if(po.getParentId().equals(p.getId())){
                //给一级指标添加末级指标得分,给末级指标添加一级指标得分
                if(p.getParentId().equals(rootId)){
                    p.setLasScore(po.getLasScore());
                    po.setFirScore(p.getFirScore());
                }
                po.setSublistName(p.getSublistName()+","+po.getSublistName());
                setParent(p, pos, rootId);
            }

        }
    }

    /**
     *
     * @param po 末级指标
     * @param pos 场景包下所有指标
     * @param rootId 场景包id，用于判断是否一级指标
     */
    private void setParent(SublistScoreVo po, SublistScoreVo parentPo, List<SublistScoreVo> pos, String rootId) {
        //第一次查找
        if(parentPo == null){
            for(SublistScoreVo p : pos){

                setSubSc(po,p,po,pos,rootId);
/*                if(po.getParentId().equals(p.getId())){
                    //根指标给一级指标添加末级指标得分,给末级指标添加一级指标得分
                    if(p.getParentId().equals(rootId)){
                        p.setLasScore(po.getLasScore());
                        po.setFirScore(p.getFirScore());
                    }
                    //末级指标拼接指标名
                    po.setSublistName(p.getSublistName()+","+po.getSublistName());
                    //查找上级指标，继续拼接
                    setParent(po,p, pos, rootId);

                }*/
            }

        }else{
            for(SublistScoreVo p : pos){

                setSubSc(parentPo,p,po,pos,rootId);

/*                //上级指标
                if(parentPo.getParentId().equals(p.getId())){
                    //根指标给一级指标添加末级指标得分,给末级指标添加一级指标得分
                    if(p.getParentId().equals(rootId)){
                        p.setLasScore(po.getLasScore());
                        po.setFirScore(p.getFirScore());
                    }
                    //末级指标拼接指标名
                    po.setSublistName(p.getSublistName()+","+po.getSublistName());
                    //查找上级指标，继续拼接
                    setParent(po,p, pos, rootId);
                }*/

            }
        }

    }


    /**
     *
     * @param p1 上级指标
     * @param p2 比对指标
     * @param p3 末级指标
     * @param pos 所有指标
     * @param rootId 场景包id
     */
    private void setSubSc(SublistScoreVo p1, SublistScoreVo p2, SublistScoreVo p3, List<SublistScoreVo> pos, String rootId){
        //上级指标
        if(p1.getParentId().equals(p2.getId())){
            //根指标给一级指标添加末级指标得分,给末级指标添加一级指标得分
            if(p2.getParentId().equals(rootId)){
                p2.setLasScore(p3.getLasScore());
                p3.setFirScore(p2.getFirScore());
            }
            //末级指标拼接指标名
            p3.setSublistName(p2.getSublistName()+","+p3.getSublistName());
            //查找上级指标，继续拼接
            setParent(p3,p2, pos, rootId);
        }


    }


/*
    *//**
     * 查询所有指标和得分
     * @param
     *//*
    private void selectScenePackageSubListAndSetScore(String pId, String scenePackageId, List<List<String>> resultArr, List<ScenePackageSubListVO>  single){

        //首次，初始化数据以及赋值
        if(isEmpty(single)){
            //单个级联，按最后一级到第一级指标顺序添加
            single= new ArrayList<>();
            //查询所有指标最后一级
            List<ScenePackageSubListVO> vos = simulationProjectMapper.selectSubListByPid(scenePackageId);
            for(ScenePackageSubListVO vo : vos){
                selectSceneSubList(vo, pId, single, resultArr);
            }
        }
        //继续添加数据
        else{
            //查询父级节点，只有一个
            ScenePackageSubListVO vo = simulationProjectMapper.selectsublistBySublistId(scenePackageId);
            //如果是根节点，不再查询，组装返回数据
            if(vo.getSublistId().equals(scenePackageId)){
                if(isEmpty(resultArr)){
                    //初始化返回数据
                    resultArr = new ArrayList<>();
                    List<String> singleList = new ArrayList<>();
                    //反转list,正序添加（从一级开始）
                    Collections.reverse(single);

                    for(int i=0;i<single.size();i--){
                        if(i == single.size()-1){
                            singleList.add(String.valueOf(single.get(i).getSceneNum()));
                            singleList.add(String.valueOf(single.get(i).getNotStandardSceneNum()));
                            singleList.add(String.valueOf(single.get(i).getScore()));

                            singleList.add(String.valueOf(single.get(i).getScore()));

                        }
                        singleList.add(single.get(i).getSublistName());
                    }






                }

            }else{
                selectSceneSubList(vo, pId, single, resultArr);
            }
        }

    };*/

/*    private void selectSceneSubList(ScenePackageSubListVO vo, String pId, List<ScenePackageSubListVO>  single, List<List<String>> resultArr){
        //获取未达标个数，得分，总分
        setScore(vo,pId);
        //放入集合
        single.add(vo);
        //递归查询上级指标
        selectScenePackageSubListAndSetScore(pId, vo.getParentId(), resultArr, single);
    }*/

    private void setScore(ScenePackageSubListVO vo, String pId){
        setLastTargetScore(vo,pId);
        setSceneScore(vo,pId);
    }

    private void setSceneScore(ScenePackageSubListVO vo, String pId){
        String sceneNaturalIds = vo.getSceneNaturalIds();
        String sceneTrafficIds = vo.getSceneTrafficIds();
        String sceneStatueIds = vo.getSceneStatueIds();
        List<SceneScoreVo> sceneScoreVos = new ArrayList<>();
        List<SceneScoreVo> NaturalSceneScoreVos = setSceneScore(vo, sceneNaturalIds, DictConstants.SCENE_NATURAL, pId);
        List<SceneScoreVo> StatueSceneScoreVos = setSceneScore(vo, sceneStatueIds, DictConstants.SCENE_STANDARD, pId);
        List<SceneScoreVo> TrafficSceneScoreVos = setSceneScore(vo, sceneTrafficIds, DictConstants.SCENE_ACCIDENT, pId);
        ///TODO 暂不支持泛化场景

        //合成一个list
        if(!isEmpty(NaturalSceneScoreVos)){
            sceneScoreVos.addAll(NaturalSceneScoreVos);
        }
        if(!isEmpty(StatueSceneScoreVos)){
            sceneScoreVos.addAll(StatueSceneScoreVos);
        }
        if(!isEmpty(TrafficSceneScoreVos)){
            sceneScoreVos.addAll(TrafficSceneScoreVos);
        }
        vo.setSceneScoreList(sceneScoreVos);
    }
    private void setLastTargetScore(ScenePackageSubListVO vo, String pId){
        SimulationMptLastTargetScorePo poParam = new SimulationMptLastTargetScorePo();
        poParam.setPId(pId);
        poParam.setTarget(vo.getSublistId());
        SimulationMptLastTargetScorePo po = simulationMptLastTargetScoreMapper.selectLastTargetScore(poParam);
        if(po != null){
            vo.setNotStandardSceneNum(po.getNotStandardSceneNum());
            vo.setScore(po.getScore());
            vo.setScoreExplain(po.getScoreExplain());
        }

    }

    private void setFirstTargetScore(ScenePackageSubListVO vo, String pId){
        SimulationMptFirstTargetScorePo poParam = new SimulationMptFirstTargetScorePo();
        poParam.setPId(pId);
        poParam.setTarget(vo.getSublistId());
        SimulationMptFirstTargetScorePo po = simulationMptFirstTargetScoreMapper.selectFirstTargetScore(poParam);
        if(po != null){
            vo.setScore(po.getScore());
        }

    }

    private List<SceneScoreVo> setSceneScore(ScenePackageSubListVO vo,String sceneIds, String sceneType, String pId){
        if(isEmpty(sceneIds)){
            return null;
        }
        String[] sceneIdArr = sceneIds.split(",");
        SceneScoreParam sceneScoreParam = new SceneScoreParam();
        sceneScoreParam.setPId(pId);
        sceneScoreParam.setLastTargerId(vo.getSublistId());
        sceneScoreParam.setSceneType(sceneType);
        sceneScoreParam.setSceneIds(Arrays.asList(sceneIdArr));
        List<SceneScoreVo> sceneScoreVos = simulationProjectTaskMapper.selectSceneScoreByIds(sceneScoreParam);

        for(SceneScoreVo s : sceneScoreVos){
            /*if(SceneTypeEnum.SCENE_NATURAL.getCode().equals(sceneType)){
                SceneBaseInfoVo sceneBaseInfoVo = simulationProjectMapper.selectSceneNatural(s.getSceneId());
                if(sceneBaseInfoVo != null){
                    s.setSceneName(sceneBaseInfoVo.getNaturalName());
                }
            }else if(SceneTypeEnum.SCENE_STANDARD.getCode().equals(sceneType)){
                SceneBaseInfoVo sceneBaseInfoVo = simulationProjectMapper.selectSceneStandardsRegulations(s.getSceneId());
                if(sceneBaseInfoVo != null){
                    s.setSceneName(sceneBaseInfoVo.getSceneName());
                }
            }else if(SceneTypeEnum.SCENE_ACCIDENT.getCode().equals(sceneType)){
                SceneBaseInfoVo sceneBaseInfoVo = simulationProjectMapper.selectSceneAccidentById(s.getSceneId());
                if(sceneBaseInfoVo != null){
                    s.setSceneName(sceneBaseInfoVo.getSceneName());
                }
            }else if(SceneTypeEnum.SCENE_GENERAL.getCode().equals(sceneType)){
                ///TODO 泛化场景暂不支持

            }*/
            s.setSceneType(SceneTypeEnum.getState(sceneType));
        }

        return sceneScoreVos;

    }

    private String getEvaluationLevelReport(Double algorithmScore){
        String evaluationLevelReport = "";
        if(algorithmScore >= 90){
            evaluationLevelReport = DictConstants.REPORT_LEVEL_G;
        }else if(algorithmScore >= 80 && algorithmScore < 90){
            evaluationLevelReport = DictConstants.REPORT_LEVEL_A;
        }else if(algorithmScore >= 70 && algorithmScore < 80){
            evaluationLevelReport = DictConstants.REPORT_LEVEL_M;
        }else if(algorithmScore < 70){
            evaluationLevelReport = DictConstants.REPORT_LEVEL_P;
        }
        return evaluationLevelReport;
    }

    /**
     * 获取二级场景下所有得分不为0的场景数量
     * @param sublistId
     * @param pId
     * @return
     */
    private Integer getSetScoreNum(String sublistId,String pId){
        SceneScoreVo vo = selectScoreNum(null, null, true, pId, sublistId);
        return vo.getNum();
    }


    /**
     * 获取一级指标下所有得分不为0的场景总数
     * @param resultVo 返回值，返回统计数量，首次传null
     * @param parentVoList 首次传null
     * @param isRoot 首次传true
     * @param pId 项目表id，用于统计得分使用
     * @param sublistId 指标id（用户查询指标下的所有场景）
     * @return
     */
    private SceneScoreVo selectScoreNum(SceneScoreVo resultVo, List<ScenePackageSubListVO> parentVoList,
                                                                                 boolean isRoot, String pId,String sublistId){
        if(isRoot){
            //初始化返回值
            resultVo = new SceneScoreVo();
            resultVo.setNum(0);

            //查找下一级指标
            parentVoList = simulationProjectMapper.selectSubSceneByPid(sublistId);
            if(!isEmpty(parentVoList)){
                selectScoreNum(resultVo, parentVoList, false, pId,null);
            }else{
                //当前指标为最后一级指标，获取当前指标，统计得分场景数
                setScoreNum(sublistId, pId, resultVo);
            }

        }else{

            //获取子节点集合
            for(ScenePackageSubListVO pvo : parentVoList){

                List<ScenePackageSubListVO> cvoList = simulationProjectMapper.selectSubSceneByPid(pvo.getSublistId());
                if(!isEmpty(cvoList)){
                    //存入父节点集合中
                    pvo.setChildScenePackageSubListVOList(cvoList);
                    //继续查找下一节点
                    selectScoreNum(resultVo, cvoList,false, pId, null);
                }else{
                    //没有子节点；获取最后一级的得分场景数量
                    setScoreNum(pvo.getSublistId(), pId, resultVo);

                }

            }
        }

        return resultVo;
    }

    private void setScoreNum(String sublistId, String pId, SceneScoreVo resultVo){
        SimulationMptSceneScorePo po = new SimulationMptSceneScorePo();
        po.setPId(pId);
        po.setLastTargerId(sublistId);
        SceneScoreVo sceneScoreVo = simulationProjectTaskMapper.selectSceneScoreNumQuery(po);
        resultVo.setNum(resultVo.getNum()+sceneScoreVo.getNum());
    }


    /**
     * 计算评测等级
     * @return
     */
    private String getEvaluationLevel(SimulationManualProjectPo po){
        /*
        汇总测试得分(计算评级)计算方法：（每一项一级指标的测试得分*测试权重）累加
         */
        List<ScenePackageSubListVO> scenePackageSubListVOS = simulationProjectMapper.selectSubSceneByPid(po.getScene());

        String evaluationLevelReport = "";
        if(!isEmpty(scenePackageSubListVOS)){
            //汇总数据初始化
            Double totalScore = 0D;

            for(ScenePackageSubListVO v : scenePackageSubListVOS){
                String weight = v.getWeight();
                Double weightDouble = Double.valueOf(weight);
                SimulationMptFirstTargetScorePo simulationMptFirstTargetScorePo = new SimulationMptFirstTargetScorePo();
                simulationMptFirstTargetScorePo.setPId(po.getId());
                simulationMptFirstTargetScorePo.setTarget(v.getSublistId());
                //单个二级指标得分
                SimulationMptFirstTargetScorePo simulationMptFirstTargetScorePo1 = simulationMptFirstTargetScoreMapper.selectFirstTargetScore(simulationMptFirstTargetScorePo);
                if(simulationMptFirstTargetScorePo1 == null){
                    return evaluationLevelReport;
                }
                Double score = simulationMptFirstTargetScorePo1.getScore();
                totalScore += score*(weightDouble/100);
            }

            evaluationLevelReport = getEvaluationLevelReport(saveTwoDecimalPlaces(totalScore));
            return evaluationLevelReport;
        }

        return evaluationLevelReport;

    }

    private void setCloums(Integer i, List<Map<String, String>> cloums){
        if(0 == i){
            Map<String, String> map = new HashMap<>();
            map.put("label","一级指标");
            map.put("prop","sublistName1");
            cloums.add(map);
        }else if(1 == i){
            Map<String, String> map = new HashMap<>();
            map.put("label","二级指标");
            map.put("prop","sublistName2");
            cloums.add(map);
        }else if(2 == i){
            Map<String, String> map = new HashMap<>();
            map.put("label","三级指标");
            map.put("prop","sublistName3");
            cloums.add(map);
        }else if(3 == i){
            Map<String, String> map = new HashMap<>();
            map.put("label","四级指标");
            map.put("prop","sublistName4");
            cloums.add(map);
        }else if(4 == i){
            Map<String, String> map = new HashMap<>();
            map.put("label","五级指标");
            map.put("prop","sublistName5");
            cloums.add(map);
        }else if(5 == i){
            Map<String, String> map = new HashMap<>();
            map.put("label","六级指标");
            map.put("prop","sublistName6");
            cloums.add(map);
        }
    }
    //运行项目统计-折线图
    @Override
    public ResponseBodyVO selectRunProjectBySy() {
        List<RunProjectVO> list=new ArrayList();
        for(Integer i=6;i>-1;i--) {
            RunProjectVO vo=new RunProjectVO();
            Map<String,Object> params = TimeUtil.getPastTime(i);
            params.put("createUserId", AuthUtil.getCurrentUserId());
            Integer num = simulationProjectMapper.selectRunProjectBySy(params);
            vo.setToDate(params.get("toDate").toString());
            vo.setNum(num);
            list.add(vo);
        }
         return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,list);
    }
    //运行任务统计-折线图
    @Override
    public ResponseBodyVO selectRunTaskBySy() {
        List<RunProjectVO> list=new ArrayList();
        for(Integer i=6;i>-1;i--) {
            RunProjectVO vo=new RunProjectVO();
            Map<String,Object> params = TimeUtil.getPastTime(i);
            params.put("createUserId", AuthUtil.getCurrentUserId());
            Integer num = simulationProjectTaskMapper.selectRunTaskBySy(params);
            vo.setToDate(params.get("toDate").toString());
            vo.setNum(num);
            list.add(vo);
        }
        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,list);
    }
    //任务运行状态统计-饼图
    @Override
    public ResponseBodyVO selectRunTaskByState() {
            Map<String,Object> params =new HashMap<>();
            params.put("createUserId", AuthUtil.getCurrentUserId());
        List<Map> list=   simulationProjectTaskMapper.selectRunTaskByState(params);

        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS, list);
    }
    //项目运行状态统计-饼图
    @Override
    public ResponseBodyVO selectRunProjectByState() {
        Map<String,Object> params =new HashMap<>();
        params.put("createUserId", AuthUtil.getCurrentUserId());
        //10：未执行，20：执行中，30：执行完成，40：已中止
        List<Map> list=simulationProjectMapper.selectRunProjectByState(params);
        if(list!=null&&list.size()>0){
            for (Map map:list) {
            String nowRunState=map.get("nowRunState").toString();
             map.put("nowRunState",  getDictName(DictConstants.PROJECT_RUN_STATE, nowRunState));
            }
        }
        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS, list);
    }
    //评测等级分布-饼图
    @Override
    public ResponseBodyVO selectEvaluationLevel() {
        Map<String,Object> params =new HashMap<>();
        params.put("createUserId", AuthUtil.getCurrentUserId());
        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS, simulationProjectMapper.selectEvaluationLevel(params));
    }

    @Override
    public void exportProjectReport(SimulationManualProjectParam param) {
        Document document = new Document(PageSize.A4);
        OutputStream os = null;
        try {
            HttpServletResponse response = ((ServletRequestAttributes)RequestContextHolder.getRequestAttributes()).getResponse();

            //获取要生成的数据
            ResponseBodyVO bodyVO = selectProjectReportById(param);
            ProjectReportVo vo = (ProjectReportVo)bodyVO.getInfo();

            //下载
            String fileName=vo.getProjectId()+"_"+vo.getProjectName();
            response.setContentType("application/x-download");
            response.addHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8") + ".pdf");
            os = new BufferedOutputStream(response.getOutputStream());

            //监听生成pdf数据
            PdfWriter.getInstance(document, os);

            // 以下生成PDF
            BaseFont font = BaseFont.createFont("STSong-Light", "UniGB-UCS2-H", BaseFont.NOT_EMBEDDED);

            //标题
            document.open();
            Font titleFont = new Font(font, 20); //字体
            Paragraph titleElements = new Paragraph("评价报告",titleFont);
            titleElements.setAlignment(Element.ALIGN_CENTER); //居中
            document.add(titleElements);


            //算法名称、算法得分、测试等级
            Paragraph algorithmNanmeElements = new Paragraph(
                    50,
                    "          算法名称："+vo.getAlgorithmName()
                            +"         算法得分："+vo.getAlgorithmScore()
                            +"         测试等级："+vo.getEvaluationLevel(),
                    defaultFont(font));
            algorithmNanmeElements.setAlignment(Element.ALIGN_LEFT); //靠左
            document.add(algorithmNanmeElements);


            //算法描述
            Paragraph algorithmDescribeFontElements = new Paragraph(
                    30,
                    "          算法描述："+vo.getAlgorithmDescribe(),
                    defaultFont(font));
            algorithmDescribeFontElements.setAlignment(Element.ALIGN_LEFT); //居中
            document.add(algorithmDescribeFontElements);

            //算法简评
            Font algorithmEvaluateFont = new Font(font, 14); //字体
            Paragraph algorithmEvaluateElements = new Paragraph(
                    30,
                    "          算法简评："+vo.getAlgorithmEvaluation(),
                    algorithmEvaluateFont);
            algorithmEvaluateElements.setAlignment(Element.ALIGN_LEFT); //居中
            document.add(algorithmEvaluateElements);

            /*
            测试得分表
             */
            Paragraph scoreTableElements = new Paragraph(
                    50,
                    "测试得分表",
                    defaultFont(font));
            scoreTableElements.setAlignment(Element.ALIGN_CENTER); //居中
            document.add(scoreTableElements);

            setBlankLineSpacing(20,font,document);

            //表头
            PdfPTable pdfPTable = new PdfPTable(5);
            pdfPTable.setHeaderRows(1);//换页每页显示表头
            addTitleList(pdfPTable, font, new String[]{"测试项目","场景数量","测试权重（%）","测试得分","得分率"});

            //数据
            List<AlgorithmScoreVo> algorithmScoreList = vo.getAlgorithmScoreList();
            for(AlgorithmScoreVo asv : algorithmScoreList){
                String[] data = new String[]{
                        asv.getProjectName(), String.valueOf(asv.getSceneNum()), asv.getWeight(), String.valueOf(asv.getScore()), String.valueOf(asv.getScoreRatio())
                };

                addDataList(pdfPTable, font, data);
            }
            document.add(pdfPTable);

            /*
            评价等级
             */
            Paragraph levelTableElements = new Paragraph(
                    50,
                    "评价等级",
                    defaultFont(font));
            levelTableElements.setAlignment(Element.ALIGN_CENTER); //居中
            document.add(levelTableElements);

            setBlankLineSpacing(20,font,document);

            PdfPTable pdfPTable1 = new PdfPTable(5);
            addTitleList(pdfPTable1, font, new String[]{"测试项目","90<总分<100","80<总分<90","70<总分<80","0<总分<70"});
            addDataList(pdfPTable1, font, new String[]{"评价等级","优秀（G）","良好（A）","一般（M）","较差（P）",
                    "评价等级","++++++","+++++","++++","+++",});
            document.add(pdfPTable1);


            //新的一页，横向显示
            document.setPageSize(PageSize.A4.rotate());
            document.newPage();

            //指标得分列表
            Paragraph subListScoreElements = new Paragraph(
                    50,
                    "算法测试评分细则",
                    defaultFont(font));
            subListScoreElements.setAlignment(Element.ALIGN_CENTER); //居中
            document.add(subListScoreElements);

            setBlankLineSpacing(20,font,document);

            List<Map> subListScoreLiTitle = vo.getSubListScoreLiTitle();
            int size = subListScoreLiTitle.size();
            PdfPTable pdfPTable2 = new PdfPTable(size);
            pdfPTable2.setHeaderRows(1);
            String[] slt = new String[size];
            for(int i=0; i<size; i++){
                slt[i] = subListScoreLiTitle.get(i).get("label").toString();
            }
            addTitleList(pdfPTable2, font, slt);

            List<SubScListVo> subListScoreLi = vo.getSubListScoreLi();

            List<String[]> list = new ArrayList<>();
            for(SubScListVo slv : subListScoreLi){
                String[] slsi = new String[size];
                for(int i=0;i<size;i++){
                    slsi[i] = "";
                }
                //目前最多支持到6级指标
                for(int i=0; i<size-4; i++){
                    if(i==0){
                        if(slv.getSublistName1() != null){
                            slsi[i] = slv.getSublistName1();
                        }

                    }else if(i==1){
                        if(slv.getSublistName2() != null){
                            slsi[i] = slv.getSublistName2();
                        }

                    }else if(i==2){
                        if(slv.getSublistName3() != null){
                            slsi[i] = slv.getSublistName3();
                        }

                    }else if(i==3){
                        if(slv.getSublistName4() != null){
                            slsi[i] = slv.getSublistName4();
                        }

                    }else if(i==4){
                        if(slv.getSublistName5() != null){
                            slsi[i] = slv.getSublistName5();
                        }

                    }else if(i==5){
                        if(slv.getSublistName6() != null){
                            slsi[i] = slv.getSublistName6();
                        }

                    }
                }
                slsi[size-4] = slv.getSceneNum();
                slsi[size-3] = slv.getNotStandardSceneNum();
                slsi[size-2] = slv.getLastScore();
                slsi[size-1] = slv.getFirstScore();

                list.add(slsi);

//                addDataList(pdfPTable2, font, slsi);
            }


            Map map = getSubScListCombineCoord(subListScoreLi, size);

            Map<String, Integer> combineCoordList = (Map<String, Integer>) map.get("combineCoordList");
            List<String> skipCoordList = (List<String>) map.get("skipCoordList");

            for(int i=0; i<list.size();i++){
                String[] strings = list.get(i);
                for(int j=0; j<strings.length; j++){
                    String coord = i+","+j;

                    //是否要跳过
                    boolean contains = skipCoordList.contains(coord);
                    if(contains){
                        continue;
                    }
                    PdfPCell pdfPCell = new PdfPCell(new Paragraph(strings[j],defaultFont(font)));

                    //是否要合并
                    Integer n =  combineCoordList.get(coord);
                    if(n != null){
                        pdfPCell.setRowspan(n);
                    }

                    pdfPTable2.addCell(pdfPCell);


                }
            }

            document.add(pdfPTable2);

            /*
            场景得分列表
             */
            Paragraph sceneScoreElements = new Paragraph(
                    50,
                    "详细得分情况",
                    defaultFont(font));
            sceneScoreElements.setAlignment(Element.ALIGN_CENTER); //居中
            document.add(sceneScoreElements);

            setBlankLineSpacing(20,font,document);

            List<Map> scoreLiTitle = vo.getSceneScoreLiTitle();
            int size2 = scoreLiTitle.size();
            PdfPTable pdfPTable3 = new PdfPTable(size2);
            pdfPTable3.setHeaderRows(1);
            String[] slt2 = new String[size2];
            for(int i=0; i<size2; i++){
                slt2[i] = scoreLiTitle.get(i).get("label").toString();
            }
            addTitleList(pdfPTable3, font, slt2);
            List<SceneScListVo> sceneScoreLi = vo.getSceneScoreLi();

            List<String[]> list2 = new ArrayList<>();

            for(SceneScListVo sslv : sceneScoreLi){
                String[] ssli = new String[size2];
                for(int i=0;i<size2;i++){
                    ssli[i] = "";
                }
                //目前最多支持到6级指标
                for(int i=0; i<size2-5; i++){
                    if(i==0){
                        if(sslv.getSublistName1() != null){
                            ssli[i] = sslv.getSublistName1();
                        }

                    }else if(i==1){
                        if(sslv.getSublistName2() != null){
                            ssli[i] = sslv.getSublistName2();
                        }

                    }else if(i==2){
                        if(sslv.getSublistName3() != null){
                            ssli[i] = sslv.getSublistName3();
                        }

                    }else if(i==3){
                        if(sslv.getSublistName4() != null){
                            ssli[i] = sslv.getSublistName4();
                        }

                    }else if(i==4){
                        if(sslv.getSublistName5() != null){
                            ssli[i] = sslv.getSublistName5();
                        }

                    }else if(i==5){
                        if(sslv.getSublistName6() != null){
                            ssli[i] = sslv.getSublistName6();
                        }

                    }
                }
                ssli[size2-5] = sslv.getSceneId();
                ssli[size2-4] = sslv.getSceneIdType();
                ssli[size2-3] = sslv.getSceneScore();
                ssli[size2-2] = sslv.getTargetEvaluate();
                ssli[size2-1] = sslv.getScoreExplain();

                list2.add(ssli);

//                addDataList(pdfPTable3, font, ssli);
            }

            Map map2 = getSceneScListCombineCoord(sceneScoreLi, size2);

            Map<String, Integer> combineCoordList2 = (Map<String, Integer>) map2.get("combineCoordList");
            List<String> skipCoordList2 = (List<String>) map2.get("skipCoordList");

            boolean b2 = false;
            int num2 = 0;
            for(int i=0; i<list2.size();i++){
                String[] strings = list2.get(i);
                for(int j=0; j<strings.length; j++){
                    String coord = i+","+j;

                    //是否要跳过
                    boolean contains = skipCoordList2.contains(coord);
                    if(contains){
                        continue;
                    }
                    PdfPCell pdfPCell = new PdfPCell(new Paragraph(strings[j],defaultFont(font)));
                    //是否要合并
                    Integer n =  combineCoordList2.get(coord);
                    if(n != null){
                        pdfPCell.setRowspan(n);
                    }

                    pdfPTable3.addCell(pdfPCell);


                }
            }

            document.add(pdfPTable3);


        } catch (Exception e){
            e.printStackTrace();

        }
        finally {
            document.close();
            try {
                os.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

    }


    /**
     * 获取所有要合并的下标、每个下标索要合并的行数、要跳过创建表格的下标
     */
    private Map getSubScListCombineCoord(List<SubScListVo> voList, int size){
        int sublistNameSize = size-4;//指标最大级数
        HashMap<String, Object> resultMap = new HashMap<>();
        Map<String, CombineCoordVo> map = new HashMap<>();
        Map<String, Integer>  combineCoordMap= new HashMap<>(); //要合并的格子下标
        List<String>  skipCoordList= new ArrayList<>(); //要跳过的格子下标

        for(int i=0; i<voList.size(); i++){
            SubScListVo vo = voList.get(i);
            if(sublistNameSize == 1){
                //一级
                String sublistName1 = vo.getSublistName1();
                if(!isEmpty(sublistName1)){
                    setCoord(map, sublistName1,i+",0", skipCoordList);
                    setCoord(map, sublistName1+"firstScore",i+","+(size-1), skipCoordList);
                }

            }else if(sublistNameSize == 2){
                //二级
                String sublistName1 = vo.getSublistName1();
                String sublistName2 = vo.getSublistName2();

                if(!isEmpty(sublistName1)){
                    setCoord(map, sublistName1,i+",0", skipCoordList);
                    setCoord(map, sublistName1+"firstScore",i+","+(size-1), skipCoordList);
                }
                if(!isEmpty(sublistName2)){
                    setCoord(map, sublistName2,i+",1", skipCoordList);
                }

            }else if(sublistNameSize == 3){
                //三级
                String sublistName1 = vo.getSublistName1();
                String sublistName2 = vo.getSublistName2();
                String sublistName3 = vo.getSublistName3();

                if(!isEmpty(sublistName1)){
                    setCoord(map, sublistName1,i+",0", skipCoordList);
                    setCoord(map, sublistName1+"firstScore",i+","+(size-1), skipCoordList);
                }
                if(!isEmpty(sublistName2)){
                    setCoord(map, sublistName2,i+",1", skipCoordList);
                }
                if(!isEmpty(sublistName3)){
                    setCoord(map, sublistName3,i+",2", skipCoordList);
                }
            }else if(sublistNameSize == 4){
                //四级
                String sublistName1 = vo.getSublistName1();
                String sublistName2 = vo.getSublistName2();
                String sublistName3 = vo.getSublistName3();
                String sublistName4 = vo.getSublistName4();

                if(!isEmpty(sublistName1)){
                    setCoord(map, sublistName1,i+",0", skipCoordList);
                    setCoord(map, sublistName1+"firstScore",i+","+(size-1), skipCoordList);
                }
                if(!isEmpty(sublistName2)){
                    setCoord(map, sublistName2,i+",1", skipCoordList);
                }
                if(!isEmpty(sublistName3)){
                    setCoord(map, sublistName3,i+",2", skipCoordList);
                }
                if(!isEmpty(sublistName4)){
                    setCoord(map, sublistName4,i+",3", skipCoordList);
                }


            }else if(sublistNameSize == 5){
                //五级
                String sublistName1 = vo.getSublistName1();
                String sublistName2 = vo.getSublistName2();
                String sublistName3 = vo.getSublistName3();
                String sublistName4 = vo.getSublistName4();
                String sublistName5 = vo.getSublistName5();

                if(!isEmpty(sublistName1)){
                    setCoord(map, sublistName1,i+",0", skipCoordList);
                    setCoord(map, sublistName1+"firstScore",i+","+(size-1), skipCoordList);
                }
                if(!isEmpty(sublistName2)){
                    setCoord(map, sublistName2,i+",1", skipCoordList);
                }
                if(!isEmpty(sublistName3)){
                    setCoord(map, sublistName3,i+",2", skipCoordList);
                }
                if(!isEmpty(sublistName4)){
                    setCoord(map, sublistName4,i+",3", skipCoordList);
                }
                if(!isEmpty(sublistName5)){
                    setCoord(map, sublistName5,i+",4", skipCoordList);
                }


            }else if(sublistNameSize == 6){
                //六级
                String sublistName1 = vo.getSublistName1();
                String sublistName2 = vo.getSublistName2();
                String sublistName3 = vo.getSublistName3();
                String sublistName4 = vo.getSublistName4();
                String sublistName5 = vo.getSublistName5();
                String sublistName6 = vo.getSublistName6();

                if(!isEmpty(sublistName1)){
                    setCoord(map, sublistName1,i+",0", skipCoordList);
                    setCoord(map, sublistName1+"firstScore",i+","+(size-1), skipCoordList);
                }
                if(!isEmpty(sublistName2)){
                    setCoord(map, sublistName2,i+",1", skipCoordList);
                }
                if(!isEmpty(sublistName3)){
                    setCoord(map, sublistName3,i+",2", skipCoordList);
                }
                if(!isEmpty(sublistName4)){
                    setCoord(map, sublistName4,i+",3", skipCoordList);
                }
                if(!isEmpty(sublistName5)){
                    setCoord(map, sublistName5,i+",4", skipCoordList);
                }
                if(!isEmpty(sublistName6)){
                    setCoord(map, sublistName6,i+",5", skipCoordList);
                }

            }



        }

        /*List<CombineCoordVo> combineCoordVoList = new ArrayList<>();

        for(String k : map.keySet()){
            combineCoordVoList.add(map.get(k));
        }*/

        for(String k : map.keySet()){
            CombineCoordVo combineCoordVo = map.get(k);
            combineCoordMap.put(combineCoordVo.getCoord(),combineCoordVo.getRepetitionNum());
        }


        resultMap.put("combineCoordList", combineCoordMap);
        resultMap.put("skipCoordList", skipCoordList);

        return resultMap;

    }


    /**
     * 获取所有要合并的下标、每个下标索要合并的行数、要跳过创建表格的下标
     */
    private Map getSceneScListCombineCoord(List<SceneScListVo> voList, int size){
        int sublistNameSize = size-5;//指标最大级数
        HashMap<String, Object> resultMap = new HashMap<>();
        Map<String, CombineCoordVo> map = new HashMap<>();
        Map<String, Integer>  combineCoordMap= new HashMap<>(); //要合并的格子下标
        List<String>  skipCoordList= new ArrayList<>(); //要跳过的格子下标

        for(int i=0; i<voList.size(); i++){
            SceneScListVo vo = voList.get(i);
            if(sublistNameSize == 1){
                //一级
                String sublistName1 = vo.getSublistName1();
                if(!isEmpty(sublistName1)){
                    setCoord(map, sublistName1,i+",0", skipCoordList);
                    setCoord(map, sublistName1+"scoreExplain",i+","+(size-1), skipCoordList);
                }

            }else if(sublistNameSize == 2){
                //二级
                String sublistName1 = vo.getSublistName1();
                String sublistName2 = vo.getSublistName2();

                if(!isEmpty(sublistName1)){
                    setCoord(map, sublistName1,i+",0", skipCoordList);
                }
                if(!isEmpty(sublistName2)){
                    setCoord(map, sublistName2,i+",1", skipCoordList);

                }
                if(!isEmpty(sublistName2)){
                    setCoord(map, sublistName2+"scoreExplain",i+","+(size-1), skipCoordList);
                }else{
                    setCoord(map, sublistName1+"scoreExplain",i+","+(size-1), skipCoordList);
                }

            }else if(sublistNameSize == 3){
                //三级
                String sublistName1 = vo.getSublistName1();
                String sublistName2 = vo.getSublistName2();
                String sublistName3 = vo.getSublistName3();

                if(!isEmpty(sublistName1)){
                    setCoord(map, sublistName1,i+",0", skipCoordList);
                }
                if(!isEmpty(sublistName2)){
                    setCoord(map, sublistName2,i+",1", skipCoordList);
                }
                if(!isEmpty(sublistName3)){
                    setCoord(map, sublistName3,i+",2", skipCoordList);

                }
                if(!isEmpty(sublistName3)){
                    setCoord(map, sublistName3+"scoreExplain",i+","+(size-1), skipCoordList);
                }else if(!isEmpty(sublistName2)){
                    setCoord(map, sublistName2+"scoreExplain",i+","+(size-1), skipCoordList);
                }else{
                    setCoord(map, sublistName1+"scoreExplain",i+","+(size-1), skipCoordList);
                }
            }else if(sublistNameSize == 4){
                //四级
                String sublistName1 = vo.getSublistName1();
                String sublistName2 = vo.getSublistName2();
                String sublistName3 = vo.getSublistName3();
                String sublistName4 = vo.getSublistName4();

                if(!isEmpty(sublistName1)){
                    setCoord(map, sublistName1,i+",0", skipCoordList);
                }
                if(!isEmpty(sublistName2)){
                    setCoord(map, sublistName2,i+",1", skipCoordList);
                }
                if(!isEmpty(sublistName3)){
                    setCoord(map, sublistName3,i+",2", skipCoordList);
                }
                if(!isEmpty(sublistName4)){
                    setCoord(map, sublistName4,i+",3", skipCoordList);
                }

                if(!isEmpty(sublistName4)){
                    setCoord(map, sublistName4+"scoreExplain",i+","+(size-1), skipCoordList);
                }else if(!isEmpty(sublistName3)){
                    setCoord(map, sublistName3+"scoreExplain",i+","+(size-1), skipCoordList);
                }else if(!isEmpty(sublistName2)){
                    setCoord(map, sublistName2+"scoreExplain",i+","+(size-1), skipCoordList);
                }else{
                    setCoord(map, sublistName1+"scoreExplain",i+","+(size-1), skipCoordList);
                }


            }else if(sublistNameSize == 5){
                //五级
                String sublistName1 = vo.getSublistName1();
                String sublistName2 = vo.getSublistName2();
                String sublistName3 = vo.getSublistName3();
                String sublistName4 = vo.getSublistName4();
                String sublistName5 = vo.getSublistName5();

                if(!isEmpty(sublistName1)){
                    setCoord(map, sublistName1,i+",0", skipCoordList);
                }
                if(!isEmpty(sublistName2)){
                    setCoord(map, sublistName2,i+",1", skipCoordList);
                }
                if(!isEmpty(sublistName3)){
                    setCoord(map, sublistName3,i+",2", skipCoordList);
                }
                if(!isEmpty(sublistName4)){
                    setCoord(map, sublistName4,i+",3", skipCoordList);
                }
                if(!isEmpty(sublistName5)){
                    setCoord(map, sublistName5,i+",4", skipCoordList);
                }

                if(!isEmpty(sublistName5)){
                    setCoord(map, sublistName5+"scoreExplain",i+","+(size-1), skipCoordList);
                }else if(!isEmpty(sublistName4)){
                    setCoord(map, sublistName4+"scoreExplain",i+","+(size-1), skipCoordList);
                }else if(!isEmpty(sublistName3)){
                    setCoord(map, sublistName3+"scoreExplain",i+","+(size-1), skipCoordList);
                }else if(!isEmpty(sublistName2)){
                    setCoord(map, sublistName2+"scoreExplain",i+","+(size-1), skipCoordList);
                }else{
                    setCoord(map, sublistName1+"scoreExplain",i+","+(size-1), skipCoordList);
                }


            }else if(sublistNameSize == 6){
                //六级
                String sublistName1 = vo.getSublistName1();
                String sublistName2 = vo.getSublistName2();
                String sublistName3 = vo.getSublistName3();
                String sublistName4 = vo.getSublistName4();
                String sublistName5 = vo.getSublistName5();
                String sublistName6 = vo.getSublistName6();

                if(!isEmpty(sublistName1)){
                    setCoord(map, sublistName1,i+",0", skipCoordList);
                }
                if(!isEmpty(sublistName2)){
                    setCoord(map, sublistName2,i+",1", skipCoordList);
                }
                if(!isEmpty(sublistName3)){
                    setCoord(map, sublistName3,i+",2", skipCoordList);
                }
                if(!isEmpty(sublistName4)){
                    setCoord(map, sublistName4,i+",3", skipCoordList);
                }
                if(!isEmpty(sublistName5)){
                    setCoord(map, sublistName5,i+",4", skipCoordList);
                }
                if(!isEmpty(sublistName6)){
                    setCoord(map, sublistName6,i+",5", skipCoordList);
                    setCoord(map, sublistName1+"scoreExplain",i+","+(size-1), skipCoordList);
                }

                if(!isEmpty(sublistName6)){
                    setCoord(map, sublistName6+"scoreExplain",i+","+(size-1), skipCoordList);
                }else if(!isEmpty(sublistName5)){
                    setCoord(map, sublistName5+"scoreExplain",i+","+(size-1), skipCoordList);
                }else if(!isEmpty(sublistName4)){
                    setCoord(map, sublistName4+"scoreExplain",i+","+(size-1), skipCoordList);
                }else if(!isEmpty(sublistName3)){
                    setCoord(map, sublistName3+"scoreExplain",i+","+(size-1), skipCoordList);
                }else if(!isEmpty(sublistName2)){
                    setCoord(map, sublistName2+"scoreExplain",i+","+(size-1), skipCoordList);
                }else{
                    setCoord(map, sublistName1+"scoreExplain",i+","+(size-1), skipCoordList);
                }

            }



        }

        /*List<CombineCoordVo> combineCoordVoList = new ArrayList<>();

        for(String k : map.keySet()){
            combineCoordVoList.add(map.get(k));
        }*/

        for(String k : map.keySet()){
            CombineCoordVo combineCoordVo = map.get(k);
            combineCoordMap.put(combineCoordVo.getCoord(),combineCoordVo.getRepetitionNum());
        }

        resultMap.put("combineCoordList", combineCoordMap);
        resultMap.put("skipCoordList", skipCoordList);

        return resultMap;

    }





    private void setCoord(Map<String, CombineCoordVo> map, String key, String coord, List<String>  skipCoordList){
        CombineCoordVo value = map.get(key);
        if(value != null){
            //坐标不变，合并行数+1
            value.setRepetitionNum(value.getRepetitionNum()+1);
            skipCoordList.add(coord);
        }else{
            //新增
            CombineCoordVo combineCoordVo = new CombineCoordVo();
            combineCoordVo.setCoord(coord);
            combineCoordVo.setRepetitionNum(1);
            map.put(key,combineCoordVo);

        }
    }

    @Override
    public String selectProjectReportIdByAlgorithmId(String algorithmId) {
        SimulationManualProjectPo po = simulationProjectMapper.selectProjectReportIdByAlgorithmId(algorithmId);
        if(StringUtil.isNotEmpty(po)){
            return po.getId();
        }
        return null;
    }

    /**
     * 换行
     * @param lineNum 换行数量
     * @param document
     */
    private void lineFeed(int lineNum, Document document) throws DocumentException {
        for(int i=0;i<lineNum;i++){
            document.add(Chunk.NEWLINE);
        }
    }

    private void setBlankLineSpacing(int lineSpacingIndex, BaseFont font, Document document) throws DocumentException {
        Paragraph elements2 = new Paragraph(
                lineSpacingIndex,
                " ",
                defaultFont(font));
        document.add(elements2);
    }

    /**
     * 默认字体
     * @param font
     * @return
     */
    private Font defaultFont(BaseFont font){
        return new Font(font, 14);
    }

    /**
     * 添加表头
     * @param pdfPTable
     * @param font
     * @param titleList
     */
    private void addTitleList(PdfPTable pdfPTable, BaseFont font, String[] titleList){
        for(String title : titleList){
            PdfPCell pdfPCell = new PdfPCell(new Paragraph(title,defaultFont(font)));
            pdfPCell.setBackgroundColor(BaseColor.LIGHT_GRAY);
            pdfPTable.addCell(pdfPCell);

        }

    }

    /**
     * 添加数据
     * @param pdfPTable
     * @param font
     * @param dataList
     */
    private void addDataList(PdfPTable pdfPTable, BaseFont font, String[] dataList){
        for(String data : dataList){
            PdfPCell pdfPCell = new PdfPCell(new Paragraph(data,defaultFont(font)));
            pdfPTable.addCell(pdfPCell);

        }

    }


    private void insertOtherAlgorithm(SimulationManualProjectParam param){
        if(ProjectConstants.SY_ALGORITHM_TYPE.equals(param.getAlgorithmType())){
            AlgorithmPO po = new AlgorithmPO();
            po.setId(param.getAlgorithm());
            po.setAlgorithmName(param.getAlgorithmName());
            po.setUploadMode(ProjectConstants.SY_ALGORITHM_TYPE);
            po.setIsDeleted("0");
            AlgorithmPO po1 = simulationProjectMapper.selectalgorithmByQuery(po);
            if(po1 == null){
                simulationProjectMapper.insertOtherAlgorithm(po);
            }else{
                if(!po1.getAlgorithmName().equals(param.getAlgorithmName())){
                    simulationProjectMapper.updateOtherAlgorithm(po);
                }
            }

        }


    }
}
