package com.css.simulation.resource.scene.service;

import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.param.scene.ScenePackageParam;
import api.common.pojo.po.scene.ScenePackagePO;
import api.common.pojo.po.scene.ScenePackageSublistPO;
import api.common.pojo.vo.scene.ScenePackageNewVO;
import api.common.pojo.vo.scene.ScenePackageSublistVO;
import api.common.util.ObjectUtil;
import api.common.util.StringUtil;
import api.common.util.TimeUtil;
import com.css.simulation.resource.common.utils.AuthUtil;
import com.css.simulation.resource.scene.mapper.ScenePackageMapper;
import com.css.simulation.resource.scene.mapper.ScenePackageSublistMapper;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import java.util.*;

@Slf4j
@Service
public class ScenePackageService {

    @Resource
    private ScenePackageMapper scenePackageMapper;
    @Resource
    private ScenePackageSublistMapper scenePackageSublistMapper;

    /**
     * 测试包数据录入大保存：
     */
    @SneakyThrows
    public ResponseBodyVO<String> saveScenePackage(List<ScenePackageSublistVO> params) {
        List<ScenePackageSublistVO> listSublictTree = new ArrayList<>();
        ScenePackagePO po = new ScenePackagePO();
        for (ScenePackageSublistVO vo : params) {
            po.setPackageName(vo.getSublistName());
            po.setCreateUserId(AuthUtil.getCurrentUserId());
            po.setShare("0");
            List<ScenePackageNewVO> listName = scenePackageMapper.queryByPackageName(po);
            if (listName != null && listName.size() > 0) {
                return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE, "测试包名称packName=" + po.getPackageName() + "已存在，请重新命名");
            }
            po.setWeight(vo.getWeight());
            po.setRemarks(vo.getRemarks());
            po.setSceneNum(vo.getSceneNum());
            po.setTreeNode(vo.getTreeNode());
            listSublictTree = vo.getChildren();
        }
        po.setCreateTime(TimeUtil.getNowForMysql());
        po.setModifyTime(TimeUtil.getNowForMysql());
        po.setPackageCode(StringUtil.getRandomCode());
        String uuId = StringUtil.getRandomUUID();
        po.setPackageId(uuId);
        po.setIsDeleted("0");
        po.setCreateUserId(AuthUtil.getCurrentUserId());

        List<ScenePackageSublistPO> poList = new ArrayList<>();
        List<ScenePackageSublistPO> treeList = instShareListTree(listSublictTree, uuId, uuId, poList, "0",1);
       //校验指标名称唯一
        if (treeList == null || treeList.size() < 1) {
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE, "指标数据为空，请检查");
        }
        if(treeList!=null && treeList.size()>1){
            Set set=new HashSet();
        Map map=new HashMap<>();
            for (ScenePackageSublistPO vo: treeList ) {
              Object sublistName=  map.get(vo.getSublistName());
              if(ObjectUtil.isNull(sublistName)){
                  map.put(vo.getSublistName(),vo.getSublistName());
              }else{
                      set.add(vo.getSublistName());
              }
            }
            if(ObjectUtil.isNotNull(set)){
                return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE, "该条数据中指标名称:"+set+"重复，请检查");
            }
        }
        scenePackageMapper.saveScenePackage(po);
        scenePackageSublistMapper.saveScenePackageSublistAll(treeList);
        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS);
    }

    //查询测试包主表列表
    @SneakyThrows
    public List<ScenePackagePO> queryScenePackageList(ScenePackageParam params) {
        List<ScenePackagePO> list = scenePackageMapper.queryScenePackageList(params);
        return list;
    }

    //修改主表数据
    @SneakyThrows
    public ResponseBodyVO<String> updateScenePackage(List<ScenePackageSublistVO> params) {
        List<ScenePackageSublistVO> listSublictTree = new ArrayList<>();
        ScenePackagePO po = new ScenePackagePO();
        for (ScenePackageSublistVO vo : params) {
            //判断唯一
            po.setPackageName(vo.getSublistName());
            po.setCreateUserId(AuthUtil.getCurrentUserId());
            po.setShare("0");
            po.setPackageId(vo.getId());
            List<ScenePackageNewVO> listName = scenePackageMapper.queryByPackageName(po);
            if (listName != null && listName.size() > 0) {
                return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE, "测试包名称packName=" + po.getPackageName() + "已存在，请重新命名");
            }
            po.setWeight(vo.getWeight());
            po.setRemarks(vo.getRemarks());
            po.setSceneNum(vo.getSceneNum());
            po.setTreeNode(vo.getTreeNode());
            listSublictTree = vo.getChildren();
        }

        po.setModifyTime(TimeUtil.getNowForMysql());
        po.setModifyUserId(AuthUtil.getCurrentUserId());
        po.setIsDeleted("0");
        po.setShare("0");
        List<ScenePackageSublistPO> poList = new ArrayList<>();
        List<ScenePackageSublistPO> treeList = instShareListTree(listSublictTree, po.getPackageId(), po.getPackageId(), poList, "0",1);
        if (treeList == null || treeList.size() < 1) {
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE, "指标数据为空，请检查");
        }
        if(treeList!=null && treeList.size()>1){
            Set set=new HashSet();
            Map map=new HashMap<>();
            for (ScenePackageSublistPO vo: treeList ) {
                Object sublistName=  map.get(vo.getSublistName());
                if(ObjectUtil.isNull(sublistName)){
                    map.put(vo.getSublistName(),vo.getSublistName());
                }else{
                    set.add(vo.getSublistName());
                }
            }
            if(ObjectUtil.isNotNull(set)){
                return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE, "该条数据中指标名称:"+set+"重复，请检查");
            }
        }
        scenePackageMapper.updateScenePackage(po);
        scenePackageSublistMapper.deleteScenePackageSublist(po.getPackageId());
        scenePackageSublistMapper.saveScenePackageSublistAll(treeList);
        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS);
    }

    //删除测试包
    @SneakyThrows
    public void deleteList(ScenePackageParam params) {
        ScenePackagePO po = new ScenePackagePO();
        po.setModifyUserId(AuthUtil.getCurrentUserId());
        po.setModifyTime(TimeUtil.getNowForMysql());
        po.setIsDeleted("1");
        po.setPackageId(params.getPackageId());
        scenePackageSublistMapper.deleteScenePackageSublist(params.getPackageId());
        scenePackageMapper.deleteScenePackage(po);
    }

    //查询一个测试包的指标列表
    @SneakyThrows
    public List<ScenePackageSublistVO> queryScenePackageSublistList(ScenePackageParam params) {
        ScenePackageNewVO vo = scenePackageMapper.queryScenePackageBySublist(params.getPackageId());
        ScenePackageSublistPO po = new ScenePackageSublistPO();
        po.setRootId(params.getPackageId());
        List<ScenePackageSublistVO> scenePackageSublistVOS = scenePackageSublistMapper.queryScenePackageSublistList(po);
        //构建根节点
        ScenePackageSublistVO root = new ScenePackageSublistVO();
        root.setId(vo.getPackageId());
        root.setParentId("0");
        root.setSublistName(vo.getPackageName());
        root.setWeight(vo.getWeight());
        root.setSceneNum(vo.getSceneNum());
        root.setRemarks(vo.getRemarks());
        root.setTreeNode(vo.getTreeNode());
        scenePackageSublistVOS.add(0, root);
        //list转tree
        List<ScenePackageSublistVO> tree = listToTree(scenePackageSublistVOS);
        return tree;
    }

    /**
     * list转tree
     */
    public static List<ScenePackageSublistVO> listToTree(List<ScenePackageSublistVO> list) {
        List<ScenePackageSublistVO> treeList = new ArrayList<ScenePackageSublistVO>();
        for (ScenePackageSublistVO vo : list) {
            if ("0".equals(vo.getParentId())) {
                treeList.add(findChildren(vo, list));
            }
        }
        return treeList;
    }

    /**
     * list转tree 递归
     */
    private static ScenePackageSublistVO findChildren(ScenePackageSublistVO vo, List<ScenePackageSublistVO> list) {
        for (ScenePackageSublistVO node : list) {
            if (vo.getId().equals(node.getParentId())) {
                if (vo.getChildren() == null || vo.getChildren().size() == 0) {
                    vo.setChildren(new ArrayList<ScenePackageSublistVO>());
                }
                vo.getChildren().add(findChildren(node, list));
            }
        }
        return vo;
    }

    /**
     * 级联获取场景指标
     *
     * @param parentVoList   首次传null
     * @param isRoot         首次传true
     * @param scenePackageId 场景包id
     * @return
     */
    public List<ScenePackageSublistVO> selectScenePackageSubListTree(List<ScenePackageSublistVO> parentVoList,
                                                                     boolean isRoot, String scenePackageId) {
        ScenePackageSublistPO params = new ScenePackageSublistPO();
        params.setParentId(scenePackageId);
        if (isRoot) {
                //查找一级节点
                parentVoList = scenePackageSublistMapper.queryScenePackageSublistList(params);
                if (parentVoList != null && parentVoList.size() > 0) {
                    selectScenePackageSubListTree(parentVoList, false, null);
                }
            } else {
            //获取子节点集合
            for (ScenePackageSublistVO pvo : parentVoList) {
                params.setParentId(pvo.getId());
                List<ScenePackageSublistVO> cvoList = scenePackageSublistMapper.queryScenePackageSublistList(params);
                if (cvoList != null && cvoList.size() > 0) {
                    //存入父节点集合中
                    pvo.setChildren(cvoList);
                    //  pvo.setHasChildren(true);
                    //继续查找下一节点
                    selectScenePackageSubListTree(cvoList, false, null);
                }
            }
        }
        return parentVoList;
    }


    //分享
    @SneakyThrows
    public ResponseBodyVO<String> instShareList(ScenePackageParam params) {
        //查询测试包主表
        ScenePackagePO scenePackagePO = scenePackageMapper.queryScenePackage(params);
        ScenePackagePO po = new ScenePackagePO();
        po.setShare("1");
        po.setPackageName(scenePackagePO.getPackageName());
        List<ScenePackageNewVO> listName = scenePackageMapper.queryByPackageName(po);
        if (listName != null && listName.size() > 0) {
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE, "测试包名称packName=" + listName.get(0).getPackageName() + "的测试包已存在，请检查");
        }
        BeanUtils.copyProperties(scenePackagePO, po);
        po.setCreateTime(TimeUtil.getNowForMysql());
        po.setModifyTime(TimeUtil.getNowForMysql());
        po.setCreateUserId(AuthUtil.getCurrentUserId());
        po.setPackageCode(StringUtil.getRandomCode());
        String uuId = StringUtil.getRandomUUID();
        po.setPackageId(uuId);
        po.setIsDeleted("0");
        po.setShare("1");
        List<ScenePackageSublistPO> poList = new ArrayList<>();
        List<ScenePackageSublistVO> listSublist = selectScenePackageSubListTree(null, true, params.getPackageId());
        List<ScenePackageSublistPO> treeList = instShareListTree(listSublist, uuId, uuId, poList, "1",1);
        if(treeList!=null && treeList.size()>1){
            Set set=new HashSet();
            Map map=new HashMap<>();
            for (ScenePackageSublistPO vo: treeList ) {
                Object sublistName=  map.get(vo.getSublistName());
                if(ObjectUtil.isNull(sublistName)){
                    map.put(vo.getSublistName(),vo.getSublistName());
                }else{
                    set.add(vo.getSublistName());
                }
            }
            if(ObjectUtil.isNotNull(set)){
                return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE, "该条数据中指标名称:"+set+"重复，请检查");
            }
        }
        scenePackageMapper.saveScenePackage(po);
        scenePackageSublistMapper.saveScenePackageSublistAll(treeList);
        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS);
    }

    /**
     * 根据级联结果转化为单级list
     *
     * @param parentVoList   首次传null
     * @param scenePackageId 场景包id
     * @return
     */
    public List<ScenePackageSublistPO> instShareListTree(List<ScenePackageSublistVO> parentVoList,
                                       String scenePackageId, String rootId, List<ScenePackageSublistPO> poList
            , String share,Integer level) {
            //获取子节点集合
            for (int i = 0; i < parentVoList.size(); i++) {
                ScenePackageSublistPO params = new ScenePackageSublistPO();
                BeanUtils.copyProperties(parentVoList.get(i), params);
                params.setParentId(scenePackageId);
                params.setRootId(rootId);
                params.setSublistName(parentVoList.get(i).getSublistName());
                params.setCreateTime(TimeUtil.getNowForMysql());
                params.setModifyTime(TimeUtil.getNowForMysql());
                params.setCreateUserId(AuthUtil.getCurrentUserId());
                String uuId = StringUtil.getRandomUUID();
                params.setId(uuId);   //主键ID
                params.setIsDeleted("0");
                params.setShare(share);
                params.setPackageLevel(level);
                params.setTreeNode(parentVoList.get(i).getTreeNode());
                List<ScenePackageSublistVO> cvoList = parentVoList.get(i).getChildren();
                poList.add(params);
                if (cvoList != null && cvoList.size() > 0) {
                    //继续查找下一节点
                    int b=level+1;
                    instShareListTree(cvoList, uuId, rootId, poList, share,b);
                }
                parentVoList.remove(parentVoList.get(i));
                i--;

            }


        return poList;
    }

    //测试包名查唯一
    public List<ScenePackageNewVO> queryByPackageName(ScenePackagePO params) {
        return scenePackageMapper.queryByPackageName(params);
    }

}
