package com.css.simulation.resource.scene.service;

import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.constants.DictConstants;
import api.common.pojo.param.MinioParameter;
import api.common.pojo.param.scene.StandardsRegulationsParam;
import api.common.pojo.po.scene.SceneAccidentPO;
import api.common.pojo.po.scene.StandardsRegulationsPO;
import api.common.util.ObjectUtil;
import api.common.util.StringUtil;
import api.common.util.TimeUtil;
import com.css.simulation.resource.common.utils.AuthUtil;
import com.css.simulation.resource.feign.FileDownService;
import com.css.simulation.resource.scene.mapper.StandardsRegulationsMapper;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import feign.Response;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service
public class StandardsRegulationsService {

    @Resource
    private StandardsRegulationsMapper standardsRegulationsMapper;

    @Resource
    private FileDownService fileDownService;
    @Resource
    private SystemUserSceneService systemUserSceneService;
    /**
     * 获取标准法规场景数据：
     */
    @SneakyThrows
    public ResponseBodyVO<String> importBzFg(StandardsRegulationsParam param) {

        MinioParameter MI = new MinioParameter();
        MI.setObjectName(param.getJsonAddress());
        Response download = fileDownService.download(MI);
        String json = download.body().toString();
        ObjectMapper objMap = new ObjectMapper();
        JsonNode root = objMap.readTree(json);
        String sceneName = root.path("场景名称").asText();
        param.setSceneName(sceneName);
        List<StandardsRegulationsPO> list = standardsRegulationsMapper.queryStandardsRegulationsByName(param);
        if (list != null && list.size() > 0) {
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE, "该场景名称" + sceneName + "已存在，请检查");
        }

        StandardsRegulationsPO standardsRegulationsPO = null;
        standardsRegulationsPO = standardsRegulationsPO.builder()
                .regulationsId(StringUtil.getRandomUUID())
                .sceneName(sceneName)
                .regulationType(root.path("法规类型").asText())
                .standardType(root.path("标准类型").asText())
                .isDeleted("0")
                .build();
        // -------------------------------- 保存数据到 mysql --------------------------------
        standardsRegulationsPO.setCreateTime(TimeUtil.getNowForMysql());
        standardsRegulationsPO.setModifyTime(TimeUtil.getNowForMysql());
        standardsRegulationsPO.setCreateUserId(AuthUtil.getCurrentUserId());
        if (null != param.getJsonAddress() && !param.getJsonAddress().equals("")) {
            standardsRegulationsPO.setJsonAddress(param.getJsonAddress());
        }
        if (null != param.getXmlAddress() && !param.getXmlAddress().equals("")) {
            standardsRegulationsPO.setXmlAddress(param.getXmlAddress());
        }
        if (null != param.getXodrAddress() && !param.getXodrAddress().equals("")) {
            standardsRegulationsPO.setXodrAddress(param.getXodrAddress());
        }
        if (null != param.getOsgbAddress() && !param.getOsgbAddress().equals("")) {
            standardsRegulationsPO.setOsgbAddress(param.getOsgbAddress());
        }
        if (null != param.getVideoAddress() && !param.getVideoAddress().equals("")) {
            standardsRegulationsPO.setVideoAddress(param.getVideoAddress());
        }
        if (null != param.getVideoPreview() && !param.getVideoPreview().equals("")) {
            standardsRegulationsPO.setVideoPreview(param.getVideoPreview());
        }
        standardsRegulationsMapper.saveStandardsRegulations(standardsRegulationsPO);

        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS);
    }

    //查看列表
    public List<StandardsRegulationsPO> queryStandardsRegulationsList(StandardsRegulationsParam params) {
        if(ObjectUtil.isNotNull(params.getSceneName()) || (params.getRegulationType()!=null&&params.getRegulationType().size()>0)||ObjectUtil.isNotNull(params.getStandardType())||ObjectUtil.isNotNull(params.getStandardType())
                ||(params.getIds()!=null&&params.getIds().length>0)){
            params.setXlk("1");
        }
        if(params.getLabel()!=null&&params.getLabel().size()>0){
            params.setBq("1");
        }
        List<StandardsRegulationsPO> list =null;

        if((params.getXlk().equals("1")&&params.getBq().equals("1"))||(params.getXlk().equals("0")&&params.getBq().equals("0"))||(params.getXlk().equals("1")&&params.getBq().equals("0"))){
            list =  standardsRegulationsMapper.queryStandardsRegulationsList(params);
        }/*else if(params.getXlk().equals("1")&&params.getBq().equals("0")){
            list =  standardsRegulationsMapper.queryStandardsRegulationsList(params);
        }*/else if(params.getXlk().equals("0")&&params.getBq().equals("1")){
            list =  standardsRegulationsMapper.queryStandardsRegulationsListByBq(params);
        }


       /* if(params.getLabel()!=null&&params.getLabel().size()>0){
            if(params.getRegulationType()!=null&&params.getRegulationType().size()>0){
                List<String> regulationType= params.getRegulationType();
                regulationType.addAll(params.getLabel());
                params.setRegulationType(regulationType);
            }else{
                List<String> regulationType=new ArrayList<>();
                regulationType.addAll(params.getLabel());
                params.setRegulationType(regulationType);
            }
        }*/

        //获取权限并赋值
      /*  List<String> listType=  systemUserSceneService.querySystemUserSceneByType(DictConstants.SCENE_STANDARD);
        if(listType!=null&&listType.size()>0){
            params.setAllIds(listType);
        }else{
            List<String> listA=new ArrayList<>();
            listA.add("没有权限");
            params.setAllIds(listA);
        }*/
        return list;
    }

    public void deleteStandardsRegulations(StandardsRegulationsPO params) {
        params.setIsDeleted("1");
        params.setModifyTime(TimeUtil.getNowForMysql());
        params.setModifyUserId(AuthUtil.getCurrentUserId());
        standardsRegulationsMapper.deleteStandardsRegulations(params);
       // systemUserSceneService.deleteSystemUserSceneBySceneId(params.getRegulationsId());
    }
}
